"""Helper functions that are used in the raytracing and analysis notebooks.

- `get_nodal_points`: Calculate the object and image nodal points of an optical system using OpticStudio.
- `get_ray_input_angle`: Calculate the input angle of a ray with respect to the optical axis.
- `get_ray_output_angle`: Calculate the output angle of a ray with respect to the optical axis and a reference point.
- `InputOutputAngles`: NamedTuple to store the input and output angles of a ray.
- `get_retina_locations`: Get the retina locations for a given row in the input-output angles DataFrame.
- `euclidean_distance`: Convert two series of tuples to NumPy arrays and calculate the element-wise euclidean distance.
- `find_ellipse_intersection`: Find the intersection between a straight line (light ray) and an ellipse.
- `ellipse_arc_length`: Calculate the arc length of an ellipse between two points.
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Callable, NamedTuple

import numpy as np
import pandas as pd
import sympy
from scipy.integrate import quad
from sympy.geometry import Ellipse, Line2D, Point2D, intersection

import zospy as zp

if TYPE_CHECKING:
    from zospy.analyses.base import AnalysisResult
    from zospy.analyses.raysandspots.single_ray_trace import SingleRayTraceResult, SingleRayTraceSettings
    from zospy.zpcore import OpticStudioSystem


def _ignore_fields(oss: OpticStudioSystem, index: int | list[int], ignore=True):
    if isinstance(index, int):
        index = [index]

    for i in index:
        oss.SystemData.Fields.GetField(i).Ignore = ignore


def get_nodal_points(oss: OpticStudioSystem) -> tuple[float, float]:
    """Calculate the object and image nodal points of an optical system using OpticStudio."""
    surface_1_to_iris = sum(oss.LDE.GetSurfaceAt(i).Thickness for i in range(1, oss.LDE.StopSurface))
    surface_2_to_iris = sum(
        oss.LDE.GetSurfaceAt(i).Thickness for i in range(oss.LDE.StopSurface, oss.LDE.NumberOfSurfaces - 1)
    )

    # Only use the chief ray
    _ignore_fields(oss, range(2, oss.SystemData.Fields.NumberOfFields + 1), True)

    cardinal_points_result = zp.analyses.reports.CardinalPoints(
        surface_1=1, surface_2=oss.LDE.NumberOfSurfaces - 1
    ).run(oss)

    object_nodal_point = cardinal_points_result.data.cardinal_points.nodal_planes.object - surface_1_to_iris
    image_nodal_point = cardinal_points_result.data.cardinal_points.nodal_planes.image + surface_2_to_iris

    # Enable rays again
    _ignore_fields(oss, range(2, oss.SystemData.Fields.NumberOfFields + 1), False)

    return object_nodal_point, image_nodal_point


def get_ray_input_angle(
    df: pd.DataFrame,
    reference_surface: int = 2,
    reference_point: tuple[float, float] | None = None,
    coordinate="Y-coordinate",
):
    """Calculate the input angle of a ray with respect to the optical axis."""
    x0, y0 = (
        df.loc[reference_surface - 1, "Z-coordinate"],
        df.loc[reference_surface - 1, coordinate],
    )

    if reference_point is None:
        x1, y1 = (
            df.loc[reference_surface, "Z-coordinate"],
            df.loc[reference_surface, coordinate],
        )
    else:
        x1, y1 = reference_point

    return np.rad2deg(np.arctan2(y1 - y0, x1 - x0))


def get_ray_output_angle(
    df: pd.DataFrame,
    reference_point: tuple[float, float] = (0, 0),
    coordinate="Y-coordinate",
):
    """Calculate the output angle of a ray with respect to the optical axis and a reference point."""
    x0, y0 = reference_point
    x1, y1 = df.loc[len(df) - 1, "Z-coordinate"], df.loc[len(df) - 1, coordinate]

    return np.rad2deg(np.arctan2(y1 - y0, x1 - x0))


class InputOutputAngles(NamedTuple):
    input_angle_field: float
    input_angle_cornea: float
    input_angle_pupil: float
    output_angle_pupil: float
    output_angle_np2: float
    output_angle_retina_center: float | None = None
    output_angle_navarro_np2: float | None = None
    location_np2: float | None = None
    location_retina_center: float | None = None
    patient: int | str | None = None

    @classmethod
    def from_ray_trace_result(
        cls,
        ray_trace_result: AnalysisResult[SingleRayTraceResult, SingleRayTraceSettings],
        field_angle: float,
        np2: float,
        np2_navarro: float | None = None,
        retina_center: float | None = None,
        patient: int | None = None,
        coordinate="Y-coordinate",
    ) -> InputOutputAngles:
        real_ray_trace_data = ray_trace_result.data.real_ray_trace_data

        return cls(
            input_angle_field=field_angle,
            input_angle_cornea=get_ray_input_angle(real_ray_trace_data, reference_surface=2, coordinate=coordinate),
            input_angle_pupil=get_ray_input_angle(real_ray_trace_data, reference_surface=4, coordinate=coordinate),
            output_angle_pupil=get_ray_output_angle(real_ray_trace_data, reference_point=(0, 0), coordinate=coordinate),
            output_angle_np2=get_ray_output_angle(real_ray_trace_data, reference_point=(np2, 0), coordinate=coordinate),
            output_angle_retina_center=(
                get_ray_output_angle(
                    real_ray_trace_data,
                    reference_point=(retina_center, 0),
                    coordinate=coordinate,
                )
                if retina_center is not None
                else None
            ),
            output_angle_navarro_np2=(
                get_ray_output_angle(
                    real_ray_trace_data,
                    reference_point=(np2_navarro, 0),
                    coordinate=coordinate,
                )
                if np2_navarro is not None
                else None
            ),
            location_np2=np2,
            location_retina_center=retina_center or None,
            patient=patient,
        )


def get_retina_locations(row: pd.Series, ray_trace_data: pd.DataFrame):
    """Get the retina locations for a given row in the input-output angles DataFrame."""
    retina_ray_trace_result = ray_trace_data.query("InputAngle == @row.input_angle_field and Comment == 'Retina'")

    if len(retina_ray_trace_result) != 1 and row.input_angle_field != 0:
        raise RuntimeError(f"Got more than one result for {row.input_angle_field=}")

    return tuple(retina_ray_trace_result.iloc[0][["Z-coordinate", "Y-coordinate"]])


def euclidean_distance(column_1: pd.Series, column_2: pd.Series):
    column_1_array = np.vstack(column_1)
    column_2_array = np.vstack(column_2)

    distance = np.linalg.norm(column_2_array - column_1_array, axis=1)
    sign = np.where(column_2_array[:, 0] > column_1_array[:, 0], -1, 1)

    return sign * distance


def _build_ellipse_intersection_function() -> Callable[[float, float, float, float, float], tuple[float, float]]:
    line_x_0, angle, r_x, r_y, ellipse_x_0 = sympy.symbols("line_x_0 angle r_x r_y ellipse_x_0")

    ellipse = Ellipse(center=Point2D(ellipse_x_0, 0), hradius=r_x, vradius=r_y)

    line = Line2D(p1=Point2D(line_x_0, 0), slope=sympy.tan(angle))

    ellipse_line_intersection = intersection(ellipse, line)

    return sympy.lambdify(
        [line_x_0, angle, r_x, r_y, ellipse_x_0],
        tuple(i.coordinates for i in ellipse_line_intersection),
        modules=["numpy"],
    )


sympy_ellipse_intersection = _build_ellipse_intersection_function()


def find_ellipse_intersection(
    reference_point: float,
    angle: float,
    r_x: float,
    r_y: float,
    ellipse_center_x: float,
) -> tuple[float, float]:
    """Find the intersection between a straight line and an ellipse.

    The ellipse has horizontal (axial) radius `r_x` and vertical (radial) radius `r_y`.
    The line intersects the horizontal axis at `distance_to_center` from the ellipse center
    and has an angle `angle` with the horizontal axis.

    Parameters
    ----------
    reference_point : float
        Point of intersection between the line and the horizontal axis.
    angle : float
        Angle of the line with the horizontal axis, in radians.
    r_x : float
        Horizontal radius of the ellipse.
    r_y : float
        Vertical radius of the ellipse.
    ellipse_center_x : float
        Horizontal location of the ellipse center.

    Returns
    -------
    tuple[float, float]
        Coordinate (x, y) of the intersection between the line and the ellipse.
    """
    # Angle of 0 is an edge case that needs to be handled separately
    if angle <= 1e-3:
        return ellipse_center_x + r_x, 0

    all_intersections = sympy_ellipse_intersection(reference_point, angle, r_x, r_y, ellipse_center_x)

    if len(all_intersections) == 2:
        if (
            # x of the 1st intersection is larger than x of the 2nd intersection
            all_intersections[0][0] > all_intersections[1][0]
            # y of the first intersection is positive
            and all_intersections[0][1] >= 0
        ):
            return all_intersections[0]

        return all_intersections[1]

    if len(all_intersections) == 1:
        return all_intersections[0]

    if len(all_intersections) == 0:
        return np.nan, np.nan

    raise ValueError("More than 2 intersections found 🤔")


def _upper_ellipse(x: float, r_x: float, r_y: float) -> float:
    return r_y * np.sqrt(1 - x**2 / r_x**2)


def _upper_ellipse_arc(x: float, r_x: float, r_y: float) -> float:
    return np.sqrt(1 + r_y**2 * x**2 / (r_x**4 - r_x**2 * x**2))


def ellipse_arc_length(x1: float, x2: float, r_x: float, r_y: float, atol: float = 1e-3) -> float:
    arc_length, error = quad(lambda x: _upper_ellipse_arc(x, r_x, r_y), x1, x2)

    if error > atol:
        raise RuntimeError(f"Absolute integration error is larger than {atol}")

    return arc_length



"""Support functions for loading the ZOS-API."""

import logging
import os
import sys
import winreg

import clr

import zospy.api.constants
from zospy.api.codecs import register_codecs
from zospy.utils import clrutils

logger = logging.getLogger(__name__)


def get_zos_root():
    """Obtain the OpticStudio folder from the Windows registry.

    Returns
    -------
    filepath : str
        The root folder of the installed OpticStudio Version

    Raises
    ------
    FileNotFoundError
        If either OpticStudio cannot be found in the Windows registry or the OpticStudio root folder could
        not be obtained from the registry.
    """
    logger.info("Obtaining OpticStudio Location from Windows Registry")

    # Search for Zemax OpticStudio in the Windows Registry
    try:
        regkey = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Software\Zemax")
    except FileNotFoundError:
        logger.exception("OpticStudio cannot be found in the registry.")
        raise

    # Obtain the OpticStudio root path from the registry
    try:
        zemaxfolder, _ = winreg.QueryValueEx(regkey, "ZemaxRoot")
        logger.info(f"Found ZemaxFolder at {zemaxfolder}")
    except FileNotFoundError:
        logger.exception("The Zemax Root folder cannot be found in the registry key.")
        raise
    finally:  # Clean up
        logger.debug("Closing registry key")
        winreg.CloseKey(regkey)
        logger.debug("Registry key closed")

    return zemaxfolder


def load_zosapi_nethelper(filepath=None, *, preload=False):
    """Load the ZOSAPI_NetHelper.

    Parameters
    ----------
    filepath : str, optional
        The location of the ZOSAPI_NetHelper. If not supplied, the location will be deducted from the windows registry.
    preload : bool
        Specifies if CLR should import with preload on (True) or off (False). Defaults to False).

    Returns
    -------
    ZOSAPI_NetHelper : netModuleObject
        The ZOSAPI_NetHelper as module object
    """
    if not filepath:  # get the path through the Windows registry
        zos_root = get_zos_root()
        znh_filepath = os.path.join(zos_root, r"ZOS-API\Libraries\ZOSAPI_NetHelper.dll")
    else:
        znh_filepath = filepath

    logger.debug(f"Adding reference {znh_filepath} to clr")
    sys.path.append(os.path.basename(znh_filepath))
    clr.AddReference(znh_filepath)

    logger.debug("Importing ZOSAPI_NetHelper")
    znh = __import__("ZOSAPI_NetHelper", globals(), locals(), [], 0)
    logger.info("ZOSAPI_NetHelper imported successfully")

    return znh


def load_zosapi(zosapi_nethelper=None, zemaxdirectory=None, *, preload=False):
    """Load the ZOS-API using the ZOSAPI NetHelper DLL.

    Parameters
    ----------
    zosapi_nethelper : netModuleObject, optional
        The loaded ZOSAPI_NetHelper DLL. If None, the zemaxdirectory parameter will be used. Note that either the
        zosapi_nethelper or the zemaxdirectory has to be specified, not both.
    zemaxdirectory : str, optional
        The directory containing the Zemax DLLs. If None, the directory will be acquired through the zosapi_nethelper.
        Note that either the zosapi_nethelper or the zemaxdirectory has to be specified, not both.
    preload : bool
        Specifies if CLR should import with preload on (True) or off (False). Defaults to False).

    Returns
    -------
    zosapi : netModuleObject
        The ZOSAPI module.

    Raises
    ------
    ValueError:
        If either both or None of zosapi_nethelper and zemaxdirectory are specified.
    FileNotFoundError:
        If one of ZOSAPI_Interfaces.dll or ZOSAPI.dll cannot be found.
    """
    if zosapi_nethelper is None and zemaxdirectory is None:
        raise ValueError("Either the zosapi_nethelper or the zemaxdirectory should be specified.")
    if zosapi_nethelper is not None and zemaxdirectory is not None:
        raise ValueError("Only one of zosapi_nethelper and zemaxdirectory should be specified.")

    # Get the Zemax OpticStudio directory and add it to the path
    if zosapi_nethelper:
        logger.info("Obtaining Zemax Directory from ZOSAPI_NetHelper")
        zosapi_nethelper.ZOSAPI_Initializer.Initialize()
        zos_dir = zosapi_nethelper.ZOSAPI_Initializer.GetZemaxDirectory()
        logger.info(f"Zemax OpticStudio found at {zos_dir}")
    else:
        logger.info(f"Zemax Directory specified by user ({zemaxdirectory})")
        zos_dir = zemaxdirectory
    sys.path.append(zos_dir)

    # Register custom Python.NET codecs
    register_codecs()

    logger.info("Searching and registering ZOSAPI DLLs")
    for dll in ["ZOSAPI_Interfaces", "ZOSAPI"]:
        if clr.FindAssembly(dll):
            logger.debug(f"{dll}.dll found")
            clr.AddReference(dll)
            logger.info(f"{dll} imported to clr")
        else:
            logger.critical(f"Cannot locate {dll}.dll")
            raise FileNotFoundError(f"Cannot locate {dll}.dll in {zos_dir}")

    logger.debug("Checking content of ZOSAPI_Interfaces.dll")
    content = clrutils.reflect_dll_content(os.path.join(zos_dir, "ZOSAPI_Interfaces.dll"))

    logger.debug("Loading ZOSAPI")
    zosapi = __import__("ZOSAPI", globals(), locals(), [], 0)
    logger.debug("ZOSAPI loaded")

    logger.debug("Loading nested namespaces")
    for nsp in content["namespaces"]:
        if nsp == "ZOSAPI":
            continue
        __import__(nsp, globals(), locals(), [], 0)
        logger.debug(f"Nested namespace {nsp} preloaded")

    zospy.api.constants._construct_from_zosapi_and_enumkeys(zosapi, content["enums"])  # noqa

    return zosapi



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from typing import overload

from zospy.api._ZOSAPI import IOpticalSystem
from zospy.api._ZOSAPI.Common import IMatrixData, IVectorData
from zospy.api._ZOSAPI.SystemData import ISDRayAimingData
from zospy.api._ZOSAPI.Tools import ISystemTool
from zospy.api._ZOSAPI.Tools.FileManager import IFile

__all__ = (
    "ACISExportVersion",
    "ArchiveFileStatus",
    "CADAngularToleranceType",
    "CADFileType",
    "CADToleranceType",
    "DataTypes",
    "EntryCompressionModes",
    "IComputeRMSSpotSize",
    "ICovarianceData",
    "ICreateArchive",
    "ICreateProject",
    "IDataDictionary",
    "IDataEntry",
    "IExportCAD",
    "IHullTools",
    "ILensCatalogLens",
    "ILensCatalogs",
    "IPoint3D",
    "IPointList3D",
    "IPointTools",
    "IQuickAdjust",
    "IQuickFocus",
    "IRayAimingWizard",
    "IRestoreArchive",
    "IRestoreProject",
    "IScale",
    "LensShape",
    "LensType",
    "QuickAdjustCriterion",
    "QuickAdjustType",
    "QuickFocusCriterion",
    "RayPatternType",
    "ScaleToUnits",
    "SplineSegmentsType",
    "ZemaxFileTypes",
)

class ACISExportVersion:
    Current = 0
    V25 = 1
    V26 = 2
    V27 = 3
    V28 = 4
    V29 = 5
    V30 = 6

class ArchiveFileStatus:
    Okay = 0
    InvalidVersion = -3
    InvalidFile = -2
    UnableToOpen = -1

class CADAngularToleranceType:
    Low = 0
    Medium = 1
    High = 2
    Presentation = 3

class CADFileType:
    IGES = 0
    STEP = 1
    SAT = 2
    STL = 3
    SAB = 4
    ASAT = 5
    ASAB = 6
    MODEL = 7
    CATPART = 8
    CATPRODUCT = 9
    XCGM = 10
    ZMO = 11
    XT = 12
    XB = 13
    PRC = 14
    JT = 15
    N3MF = 16
    U3D = 17
    VRML = 18
    OBJ = 19

class CADToleranceType:
    N_TenEMinus4 = 0
    N_TenEMinus5 = 1
    N_TenEMinus6 = 2
    N_TenEMinus7 = 3

class DataTypes:
    NotSet = 0
    Boolean = 1
    Integer = 2
    IntegerArray = 3
    IntegerMatrix = 4
    Float = 5
    FloatArray = 6
    FloatMatrix = 7
    Double = 8
    DoubleArray = 9
    DoubleMatrix = 10
    String = 11
    StringArray = 12
    StringMatrix = 13
    ByteArray = 14
    Dictionary = 15
    Serializable = 16
    File = 17

class EntryCompressionModes:
    Auto = 0
    On = 1
    Off = 2

class IComputeRMSSpotSize(ISystemTool):
    @property
    def UseGaussianQuadrature(self) -> bool: ...
    @property
    def UseRectangularArray(self) -> bool: ...
    @property
    def NumberOfRings(self) -> int: ...
    @property
    def NumberOfArms(self) -> int: ...
    @property
    def ObscuredFration(self) -> float: ...
    @property
    def GridSampling(self) -> int: ...
    @property
    def DeleteVignetted(self) -> bool: ...
    @property
    def AssumeAxialSymmetry(self) -> bool: ...
    @AssumeAxialSymmetry.setter
    def AssumeAxialSymmetry(self, value: bool) -> None: ...
    @property
    def IgnoreLateralColor(self) -> bool: ...
    @IgnoreLateralColor.setter
    def IgnoreLateralColor(self, value: bool) -> None: ...
    @property
    def NumberOfConfigurations(self) -> int: ...
    @property
    def NumberOfFields(self) -> int: ...
    @property
    def RMSSpotRadii(self) -> list[list[float]]: ...
    def GetSpotRadius(self, configuration: int, field: int) -> float: ...
    def SetMethodGaussianQuadrature(self, numberOfRings: int, numberOfArms: int, obscuredFraction: float) -> None: ...
    def SetMethodRectangularArray(self, gridSampling: int, deleteVignetted: bool) -> None: ...

class ICovarianceData:
    @property
    def Cov(self) -> list[list[float]]: ...
    @property
    def V(self) -> list[list[float]]: ...
    @property
    def W(self) -> list[float]: ...
    def GetCovariance(self) -> IMatrixData: ...
    def GetValues(self) -> IVectorData: ...
    def GetVectors(self) -> IMatrixData: ...

class ICreateArchive(ISystemTool):
    @property
    def UseDataCompression(self) -> bool: ...
    @UseDataCompression.setter
    def UseDataCompression(self, value: bool) -> None: ...
    def GetArchiveFileName(self) -> str: ...
    def SetArchiveFileName(self, fullFileName: str) -> bool: ...

class ICreateProject(ISystemTool):
    @property
    def UseDataCompression(self) -> bool: ...
    @UseDataCompression.setter
    def UseDataCompression(self, value: bool) -> None: ...
    def GetArchiveFileName(self) -> str: ...
    def SetArchiveFileName(self, fullFileName: str) -> bool: ...

class IDataDictionary:
    @property
    def FileName(self) -> str: ...
    @property
    def FileType(self) -> ZemaxFileTypes: ...
    @property
    def FileVersion(self) -> int: ...
    @property
    def FileData1(self) -> int: ...
    @property
    def FileData2(self) -> int: ...
    @property
    def Keys(self) -> list[str]: ...
    @property
    def NumberOfEntries(self) -> int: ...
    @property
    def Entries(self) -> list[IDataEntry]: ...
    def Close(self) -> None: ...
    def CreateEntry(self, key: str) -> IDataEntry: ...
    def GetEntry(self, key: str) -> IDataEntry: ...
    def GetKey(self, keyIndex: int) -> str: ...
    def Save(self) -> None: ...

class IDataEntry:
    @property
    def DataType(self) -> DataTypes: ...
    @property
    def IsSet(self) -> bool: ...
    @property
    def Key(self) -> str: ...
    @property
    def CompressionMode(self) -> EntryCompressionModes: ...
    @CompressionMode.setter
    def CompressionMode(self, value: EntryCompressionModes) -> None: ...
    def Delete(self) -> None: ...
    def GetBooleanValue(self) -> bool: ...
    def GetByteArray(self) -> list[int]: ...
    def GetDoubleArray(self) -> list[float]: ...
    def GetDoubleMatrix(self) -> list[list[float]]: ...
    def GetDoubleValue(self) -> float: ...
    def GetFile(self, fileName: str) -> None: ...
    def GetFloatArray(self) -> list[float]: ...
    def GetFloatMatrix(self) -> list[list[float]]: ...
    def GetFloatValue(self) -> float: ...
    def GetIntegerArray(self) -> list[int]: ...
    def GetIntegerMatrix(self) -> list[list[int]]: ...
    def GetIntegerValue(self) -> int: ...
    def GetSerializableValue(self) -> Object: ...
    def GetStringArray(self) -> list[str]: ...
    def GetStringMatrix(self) -> list[list[str]]: ...
    def GetStringValue(self) -> str: ...
    def SetBooleanValue(self, val: bool) -> bool: ...
    def SetByteArray(self, val: list[int]) -> bool: ...
    def SetDoubleArray(self, val: list[float]) -> bool: ...
    def SetDoubleMatrix(self, val: list[list[float]]) -> bool: ...
    def SetDoubleValue(self, val: float) -> bool: ...
    def SetFile(self, fileName: str) -> bool: ...
    def SetFloatArray(self, val: list[float]) -> bool: ...
    def SetFloatMatrix(self, val: list[list[float]]) -> bool: ...
    def SetFloatValue(self, val: float) -> bool: ...
    def SetIntegerArray(self, val: list[int]) -> bool: ...
    def SetIntegerMatrix(self, val: list[list[int]]) -> bool: ...
    def SetIntegerValue(self, val: int) -> bool: ...
    def SetSerializableValue(self, val: Object) -> bool: ...
    def SetStringArray(self, val: list[str]) -> bool: ...
    def SetStringMatrix(self, val: list[list[str]]) -> bool: ...
    def SetStringValue(self, val: str) -> bool: ...

class IExportCAD(ISystemTool):
    @property
    def FirstSurface(self) -> int: ...
    @FirstSurface.setter
    def FirstSurface(self, value: int) -> None: ...
    @property
    def LastSurface(self) -> int: ...
    @LastSurface.setter
    def LastSurface(self, value: int) -> None: ...
    @property
    def CanChangeSurface(self) -> bool: ...
    @property
    def FirstObject(self) -> int: ...
    @FirstObject.setter
    def FirstObject(self, value: int) -> None: ...
    @property
    def LastObject(self) -> int: ...
    @LastObject.setter
    def LastObject(self, value: int) -> None: ...
    @property
    def SequentialSettingsEnabled(self) -> bool: ...
    @property
    def RayPattern(self) -> RayPatternType: ...
    @RayPattern.setter
    def RayPattern(self, value: RayPatternType) -> None: ...
    @property
    def NumberOfRays(self) -> int: ...
    @NumberOfRays.setter
    def NumberOfRays(self, value: int) -> None: ...
    @property
    def Wavelength(self) -> int: ...
    @Wavelength.setter
    def Wavelength(self, value: int) -> None: ...
    @property
    def Field(self) -> int: ...
    @Field.setter
    def Field(self, value: int) -> None: ...
    @property
    def DeleteVignetted(self) -> bool: ...
    @DeleteVignetted.setter
    def DeleteVignetted(self, value: bool) -> None: ...
    @property
    def RayLayer(self) -> int: ...
    @RayLayer.setter
    def RayLayer(self, value: int) -> None: ...
    @property
    def LensLayer(self) -> int: ...
    @LensLayer.setter
    def LensLayer(self, value: int) -> None: ...
    @property
    def DummyThickness(self) -> float: ...
    @DummyThickness.setter
    def DummyThickness(self, value: float) -> None: ...
    @property
    def SplineSegments(self) -> SplineSegmentsType: ...
    @SplineSegments.setter
    def SplineSegments(self, value: SplineSegmentsType) -> None: ...
    @property
    def FileType(self) -> CADFileType: ...
    @FileType.setter
    def FileType(self, value: CADFileType) -> None: ...
    @property
    def Tolerance(self) -> CADToleranceType: ...
    @Tolerance.setter
    def Tolerance(self, value: CADToleranceType) -> None: ...
    @property
    def AngularTolerance(self) -> CADAngularToleranceType: ...
    @AngularTolerance.setter
    def AngularTolerance(self, value: CADAngularToleranceType) -> None: ...
    @property
    def Configuration(self) -> int: ...
    @property
    def ExportDummySurfaces(self) -> bool: ...
    @ExportDummySurfaces.setter
    def ExportDummySurfaces(self, value: bool) -> None: ...
    @property
    def SurfacesAsSolids(self) -> bool: ...
    @SurfacesAsSolids.setter
    def SurfacesAsSolids(self, value: bool) -> None: ...
    @property
    def ScatterNSCRays(self) -> bool: ...
    @ScatterNSCRays.setter
    def ScatterNSCRays(self, value: bool) -> None: ...
    @property
    def SplitNSCRays(self) -> bool: ...
    @SplitNSCRays.setter
    def SplitNSCRays(self, value: bool) -> None: ...
    @property
    def UsePolarization(self) -> bool: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @property
    def OutputFileName(self) -> str: ...
    @OutputFileName.setter
    def OutputFileName(self, value: str) -> None: ...
    @property
    def AutoNameFaces(self) -> bool: ...
    @AutoNameFaces.setter
    def AutoNameFaces(self, value: bool) -> None: ...
    @property
    def FileVersion(self) -> ACISExportVersion: ...
    @FileVersion.setter
    def FileVersion(self, value: ACISExportVersion) -> None: ...
    def SetConfigurationAllAtOnce(self) -> None: ...
    def SetConfigurationAllByFile(self) -> None: ...
    def SetConfigurationAllByLayer(self) -> None: ...
    def SetCurrentConfiguration(self) -> None: ...
    def SetFieldAll(self) -> None: ...
    def SetSingleConfiguration(self, ConfigurationNumber: int) -> None: ...
    def SetWavelengthAll(self) -> None: ...

class IHullTools:
    def Compute2DHull(
        self, points: IPointList3D, planeOrigin: IPoint3D, planeX: IPoint3D, planeY: IPoint3D
    ) -> IPointList3D: ...
    def ComputeCovData(self, points: IPointList3D, center: IPoint3D) -> ICovarianceData: ...

class ILensCatalogLens:
    @property
    def Vendor(self) -> str: ...
    @property
    def LensName(self) -> str: ...
    @property
    def EFL(self) -> float: ...
    @property
    def EPD(self) -> float: ...
    @property
    def Shape(self) -> LensShape: ...
    @property
    def Type(self) -> LensType: ...
    @property
    def NumberOfElements(self) -> int: ...
    @property
    def Telecentric(self) -> bool: ...
    def InsertLensNonSeq(self) -> bool: ...
    def InsertLensSeq(self, atSurface: int, ignoreObject: bool, reverseSurfGeom: bool) -> bool: ...
    def LoadLens(self, saveIfNeeded: bool) -> None: ...
    def LoadNewLensNonSeq(self) -> IOpticalSystem: ...
    def LoadNewLensSeq(self) -> IOpticalSystem: ...

class ILensCatalogs(ISystemTool):
    @property
    def SelectedVendor(self) -> str: ...
    @SelectedVendor.setter
    def SelectedVendor(self, value: str) -> None: ...
    @property
    def UseEFL(self) -> bool: ...
    @UseEFL.setter
    def UseEFL(self, value: bool) -> None: ...
    @property
    def MinEFL(self) -> float: ...
    @MinEFL.setter
    def MinEFL(self, value: float) -> None: ...
    @property
    def MaxEFL(self) -> float: ...
    @MaxEFL.setter
    def MaxEFL(self, value: float) -> None: ...
    @property
    def UseEPD(self) -> bool: ...
    @UseEPD.setter
    def UseEPD(self, value: bool) -> None: ...
    @property
    def MinEPD(self) -> float: ...
    @MinEPD.setter
    def MinEPD(self, value: float) -> None: ...
    @property
    def MaxEPD(self) -> float: ...
    @MaxEPD.setter
    def MaxEPD(self, value: float) -> None: ...
    @property
    def IncShapeEqui(self) -> bool: ...
    @IncShapeEqui.setter
    def IncShapeEqui(self, value: bool) -> None: ...
    @property
    def IncShapeBi(self) -> bool: ...
    @IncShapeBi.setter
    def IncShapeBi(self, value: bool) -> None: ...
    @property
    def IncShapePlano(self) -> bool: ...
    @IncShapePlano.setter
    def IncShapePlano(self, value: bool) -> None: ...
    @property
    def IncShapeMeniscus(self) -> bool: ...
    @IncShapeMeniscus.setter
    def IncShapeMeniscus(self, value: bool) -> None: ...
    @property
    def IncTypeSpherical(self) -> bool: ...
    @IncTypeSpherical.setter
    def IncTypeSpherical(self, value: bool) -> None: ...
    @property
    def IncTypeGRIN(self) -> bool: ...
    @IncTypeGRIN.setter
    def IncTypeGRIN(self, value: bool) -> None: ...
    @property
    def IncTypeAspheric(self) -> bool: ...
    @IncTypeAspheric.setter
    def IncTypeAspheric(self, value: bool) -> None: ...
    @property
    def IncTypeToroidal(self) -> bool: ...
    @IncTypeToroidal.setter
    def IncTypeToroidal(self, value: bool) -> None: ...
    @property
    def NumberOfElements(self) -> int: ...
    @NumberOfElements.setter
    def NumberOfElements(self, value: int) -> None: ...
    @property
    def MatchingLenses(self) -> int: ...
    @property
    def LensesForVendor(self) -> int: ...
    def GetAllVendors(self) -> list[str]: ...
    def GetResult(self, index: int) -> ILensCatalogLens: ...
    def SetNumberOfElementsAny(self) -> None: ...

class IPoint3D:
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> None: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> None: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> None: ...
    def Add(self, p: IPoint3D) -> None: ...
    def GetDim(self, Dim: int) -> float: ...
    def GetSortedDims(self) -> list[int]: ...
    def Magnitude(self) -> float: ...
    def MagnitudeSq(self) -> float: ...
    def SetDim(self, Dim: int, Value: float) -> None: ...
    def VectorAndScalar(self) -> tuple[bool, IPoint3D, float]: ...

class IPointList3D:
    @property
    def NumberOfPoints(self) -> int: ...
    @overload
    def AddPoint(self, p: IPoint3D) -> None: ...
    @overload
    def AddPoint(self, X: float, Y: float, Z: float) -> None: ...
    def Clear(self) -> None: ...
    def GetPoint(self, Index: int) -> IPoint3D: ...
    def RemovePoint(self, Index: int) -> None: ...

class IPointTools:
    def Add(self, p1: IPoint3D, p2: IPoint3D) -> IPoint3D: ...
    def ComputeCentroid(self, points: IPointList3D) -> IPoint3D: ...
    def CreatePoint(self, x: float, y: float, z: float) -> IPoint3D: ...
    def CreatePointList(self) -> IPointList3D: ...
    def Cross(self, p1: IPoint3D, p2: IPoint3D) -> IPoint3D: ...
    def Divide(self, p: IPoint3D, s: float) -> IPoint3D: ...
    def Dot(self, p1: IPoint3D, p2: IPoint3D) -> float: ...
    def Multiply(self, p: IPoint3D, s: float) -> IPoint3D: ...
    def Subtract(self, p1: IPoint3D, p2: IPoint3D) -> IPoint3D: ...

class IQuickAdjust(ISystemTool):
    @property
    def AdjustSurface(self) -> int: ...
    @AdjustSurface.setter
    def AdjustSurface(self, value: int) -> None: ...
    @property
    def SurfaceParameter(self) -> QuickAdjustType: ...
    @SurfaceParameter.setter
    def SurfaceParameter(self, value: QuickAdjustType) -> None: ...
    @property
    def Criterion(self) -> QuickAdjustCriterion: ...
    @Criterion.setter
    def Criterion(self, value: QuickAdjustCriterion) -> None: ...
    @property
    def EvaluateSurface(self) -> int: ...
    @EvaluateSurface.setter
    def EvaluateSurface(self, value: int) -> None: ...
    def SetEvaluateSurfaceImage(self) -> None: ...

class IQuickFocus(ISystemTool):
    @property
    def UseCentroid(self) -> bool: ...
    @UseCentroid.setter
    def UseCentroid(self, value: bool) -> None: ...
    @property
    def Criterion(self) -> QuickFocusCriterion: ...
    @Criterion.setter
    def Criterion(self, value: QuickFocusCriterion) -> None: ...

class IRayAimingWizard(ISystemTool):
    def ApplyRecommendedSettings(self) -> None: ...
    def GetRecommendedSettings(self) -> ISDRayAimingData: ...

class IRestoreArchive(ISystemTool):
    @property
    def ArchiveFile(self) -> str: ...
    @property
    def NumberOfFilesInArchive(self) -> int: ...
    @property
    def ExtractAllFilesToProjectDirectory(self) -> bool: ...
    @ExtractAllFilesToProjectDirectory.setter
    def ExtractAllFilesToProjectDirectory(self, value: bool) -> None: ...
    def GetFileNameInArchive(self, fileNumber: int) -> str: ...
    def GetOutputFolder(self) -> str: ...
    def SetFileName(self, archiveFile: str) -> ArchiveFileStatus: ...
    def SetFileOverwrite(self, fileNumber: int, allowOverwrite: bool) -> None: ...
    def SetFilesAllOverwrite(self) -> None: ...
    def SetFilesNoOverwrite(self) -> None: ...
    def SetOutputFolder(self, folder: str) -> bool: ...

class IRestoreProject(ISystemTool):
    @property
    def ArchiveFile(self) -> str: ...
    @property
    def NumberOfFilesInArchive(self) -> int: ...
    @property
    def NumberOfAdditionalFiles(self) -> int: ...
    def GetAdditionalFile(self, addfileNumber: int) -> IFile: ...
    def GetFile(self, fileNumber: int) -> IFile: ...
    def GetFileNameInArchive(self, fileNumber: int) -> str: ...
    def GetOutputFolder(self) -> str: ...
    def SetAllFilesToProject(self) -> bool: ...
    def SetAllFilesToRoot(self) -> bool: ...
    def SetFileName(self, archiveFile: str) -> ArchiveFileStatus: ...
    def SetFileOverwrite(self, fileNumber: int, allowOverwrite: bool) -> None: ...
    def SetFilesAllOverwrite(self) -> None: ...
    def SetFilesNoOverwrite(self) -> None: ...
    def SetOutputFolder(self, folder: str) -> bool: ...
    def UseProjectFile(self, fileNumber: int) -> bool: ...
    def UseRootFile(self, fileNumber: int) -> bool: ...

class IScale(ISystemTool):
    @property
    def ScaleFactor(self) -> float: ...
    @ScaleFactor.setter
    def ScaleFactor(self, value: float) -> None: ...
    @property
    def FirstComponent(self) -> int: ...
    @FirstComponent.setter
    def FirstComponent(self, value: int) -> None: ...
    @property
    def LastComponent(self) -> int: ...
    @LastComponent.setter
    def LastComponent(self, value: int) -> None: ...
    @property
    def ScaleToUnit(self) -> ScaleToUnits: ...
    @ScaleToUnit.setter
    def ScaleToUnit(self, value: ScaleToUnits) -> None: ...
    @property
    def ScaleByFactor(self) -> bool: ...
    @ScaleByFactor.setter
    def ScaleByFactor(self, value: bool) -> None: ...
    @property
    def ScaleByUnits(self) -> bool: ...
    @ScaleByUnits.setter
    def ScaleByUnits(self, value: bool) -> None: ...
    @property
    def NumberOfComponents(self) -> int: ...
    def GetComponentAt(self, idx: int) -> str: ...

class LensShape:
    Unknown = 0
    Equi = 1
    Bi = 2
    Plano = 3
    Meniscus = 4

class LensType:
    Other = 0
    Spherical = 1
    GRIN = 2
    Aspheric = 3
    Toroidal = 4

class QuickAdjustCriterion:
    SpotSizeRadial = 0
    SpotSizeXOnly = 1
    SpotSizeYOnly = 2
    AngularRadial = 3
    AngularXOnly = 4
    AngularYOnly = 5

class QuickAdjustType:
    Radius = 0
    Thickness = 1

class QuickFocusCriterion:
    SpotSizeRadial = 0
    SpotSizeXOnly = 1
    SpotSizeYOnly = 2
    RMSWavefront = 3

class RayPatternType:
    XYFan = 0
    XFan = 1
    YFan = 2
    Ring = 3
    List = 4
    Random = 5
    Grid = 6
    SolidRing = 7

class ScaleToUnits:
    Millimeters = 0
    Centimeters = 1
    Inches = 2
    Meters = 3

class SplineSegmentsType:
    N_016 = 0
    N_032 = 1
    N_064 = 2
    N_128 = 3
    N_256 = 4
    N_512 = 5

class ZemaxFileTypes:
    Unknown = 0
    LMX = 1
    User = 2



import numpy as np
import pytest
from pandas.testing import assert_frame_equal

from zospy.analyses.old.mtf import (
    fft_through_focus_mtf,
    fft_through_focus_mtf_fromcfg,
    huygens_mtf,
)

pytestmark = pytest.mark.old_analyses

_FFT_THROUGH_FOCUS_MTF_MTFTYPE_EXPECTED_RETURN = {
    # The expected return does not match constants.Analysis.Settings.Mtf.MtfTypes for fft_through_focus_mtf
    "Modulation": "5",
    "Real": "6",
    "Imaginary": "7",
    "Phase": "8",
    "SquareWave": "9",
}


class TestFFTThroughFocusMTF:
    def test_can_run_fft_through_focus_mtf(self, simple_system):
        result = fft_through_focus_mtf(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = fft_through_focus_mtf(simple_system)

        assert result.from_json(result.to_json())

    def test_can_run_fft_through_focus_mtf_fromcfg(self, simple_system, cfg_file):
        result = fft_through_focus_mtf_fromcfg(simple_system, str(cfg_file))

        assert result.Data is not None

    def test_fft_through_focus_mtf_fromcfg_loads_config_correctly(self, simple_system, cfg_file):
        result = fft_through_focus_mtf_fromcfg(simple_system, str(cfg_file))

        assert result.Settings.SampleSize == "S_128x128"
        assert result.Settings.DeltaFocus == 1.2
        assert result.Settings.Frequency == 3.0
        assert result.Settings.NumberOfSteps == 9
        # assert result.Settings.Type == "Imaginary"
        assert result.Settings.UsePolarization

    @pytest.mark.parametrize(
        "sampling,deltafocus,frequency,numberofsteps,mtftype",
        [("64x64", 0.1, 0, 5, "Modulation"), ("128x128", 0.3, 3, 10, "Imaginary")],
    )
    def test_fft_through_focus_mtf_returns_correct_result(
        self, simple_system, sampling, deltafocus, frequency, numberofsteps, mtftype, expected_data
    ):
        result = fft_through_focus_mtf(
            simple_system,
            sampling=sampling,
            deltafocus=deltafocus,
            frequency=frequency,
            numberofsteps=numberofsteps,
            mtftype=mtftype,
        )

        assert np.allclose(result.Data.astype(float), expected_data.Data.astype(float), rtol=1e-3)

    @pytest.mark.parametrize(
        "sampling,deltafocus,frequency,numberofsteps,mtftype",
        [("64x64", 0.1, 0, 5, "Modulation"), ("128x128", 0.3, 3, 10, "Imaginary")],
    )
    def test_fft_through_focus_mtf_matches_reference_data(
        self, simple_system, sampling, deltafocus, frequency, numberofsteps, mtftype, reference_data
    ):
        result = fft_through_focus_mtf(
            simple_system,
            sampling=sampling,
            deltafocus=deltafocus,
            frequency=frequency,
            numberofsteps=numberofsteps,
            mtftype=mtftype,
        )

        assert np.allclose(result.Data.astype(float), reference_data.Data.astype(float), rtol=1e-3)

    @pytest.mark.parametrize(
        "mtftype",
        ["Modulation", "Real", "Imaginary", "Phase", "SquareWave"],
    )
    def test_fft_through_focus_mtf_sets_mtftype_correctly(self, simple_system, mtftype):
        result = fft_through_focus_mtf(
            simple_system,
            mtftype=mtftype,
        )

        assert result.Settings["Type"] == _FFT_THROUGH_FOCUS_MTF_MTFTYPE_EXPECTED_RETURN[mtftype]


class TestHuygensMTF:
    def test_can_run_huygens_mtf(self, simple_system):
        result = huygens_mtf(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = huygens_mtf(simple_system)

        assert result.from_json(result.to_json())

    @pytest.mark.parametrize(
        "pupil_sampling,image_sampling,image_delta,mtftype,maximum_frequency",
        [
            ("64x64", "64x64", 0.0, "Modulation", 150.0),
            ("32x32", "64x64", 1.0, "Modulation", 450.0),
            ("128x128", "128x128", 0.0, "Modulation", 314.5),
            ("32x32", "32x32", 0.0, "Modulation", 150.0),
        ],
    )
    def test_huygens_mtf_returns_correct_result(
        self, simple_system, pupil_sampling, image_sampling, image_delta, mtftype, maximum_frequency, expected_data
    ):
        result = huygens_mtf(
            simple_system,
            pupil_sampling=pupil_sampling,
            image_sampling=image_sampling,
            image_delta=image_delta,
            mtftype=mtftype,
            maximum_frequency=maximum_frequency,
        )

        assert_frame_equal(result.Data.astype(float), expected_data.Data.astype(float))

    @pytest.mark.parametrize(
        "pupil_sampling,image_sampling,image_delta,mtftype,maximum_frequency",
        [
            ("64x64", "64x64", 0.0, "Modulation", 150.0),
            ("32x32", "64x64", 1.0, "Modulation", 450.0),
            ("128x128", "128x128", 0.0, "Modulation", 314.5),
            ("32x32", "32x32", 0.0, "Modulation", 150.0),
        ],
    )
    def test_huygens_mtf_matches_reference_data(
        self, simple_system, pupil_sampling, image_sampling, image_delta, mtftype, maximum_frequency, reference_data
    ):
        result = huygens_mtf(
            simple_system,
            pupil_sampling=pupil_sampling,
            image_sampling=image_sampling,
            image_delta=image_delta,
            mtftype=mtftype,
            maximum_frequency=maximum_frequency,
        )

        assert_frame_equal(result.Data.astype(float), reference_data.Data.astype(float))



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = (
    "ICoordinateTransform",
    "IMatrixData",
    "IMetadata",
    "ISettingsData",
    "IVectorData",
    "SettingsDataType",
    "ZemaxColor",
    "ZemaxOpacity",
)

class ICoordinateTransform:
    @property
    def HasOffset(self) -> bool: ...
    @property
    def XOffset(self) -> float: ...
    @XOffset.setter
    def XOffset(self, value: float) -> None: ...
    @property
    def YOffset(self) -> float: ...
    @YOffset.setter
    def YOffset(self, value: float) -> None: ...
    @property
    def ZOffset(self) -> float: ...
    @ZOffset.setter
    def ZOffset(self, value: float) -> None: ...
    @property
    def HasRotation(self) -> bool: ...
    @property
    def R11(self) -> float: ...
    @R11.setter
    def R11(self, value: float) -> None: ...
    @property
    def R12(self) -> float: ...
    @R12.setter
    def R12(self, value: float) -> None: ...
    @property
    def R13(self) -> float: ...
    @R13.setter
    def R13(self, value: float) -> None: ...
    @property
    def R21(self) -> float: ...
    @R21.setter
    def R21(self, value: float) -> None: ...
    @property
    def R22(self) -> float: ...
    @R22.setter
    def R22(self, value: float) -> None: ...
    @property
    def R23(self) -> float: ...
    @R23.setter
    def R23(self, value: float) -> None: ...
    @property
    def R31(self) -> float: ...
    @R31.setter
    def R31(self, value: float) -> None: ...
    @property
    def R32(self) -> float: ...
    @R32.setter
    def R32(self, value: float) -> None: ...
    @property
    def R33(self) -> float: ...
    @R33.setter
    def R33(self, value: float) -> None: ...
    @property
    def RotationAnglesSpecified(self) -> bool: ...
    @RotationAnglesSpecified.setter
    def RotationAnglesSpecified(self, value: bool) -> None: ...
    @property
    def ThetaX(self) -> float: ...
    @ThetaX.setter
    def ThetaX(self, value: float) -> None: ...
    @property
    def ThetaY(self) -> float: ...
    @ThetaY.setter
    def ThetaY(self, value: float) -> None: ...
    @property
    def ThetaZ(self) -> float: ...
    @ThetaZ.setter
    def ThetaZ(self, value: float) -> None: ...
    @property
    def CentroidX(self) -> float: ...
    @CentroidX.setter
    def CentroidX(self, value: float) -> None: ...
    @property
    def CentroidY(self) -> float: ...
    @CentroidY.setter
    def CentroidY(self, value: float) -> None: ...
    @property
    def CentroidZ(self) -> float: ...
    @CentroidZ.setter
    def CentroidZ(self, value: float) -> None: ...
    def GetAngleSet(self) -> list[float]: ...
    def GetDecenterVector(self) -> list[float]: ...
    def GetMatrix_3x3(self) -> list[list[float]]: ...
    def GetMatrix_4x4(self) -> list[list[float]]: ...

class IMatrixData:
    @property
    def IsReadOnly(self) -> bool: ...
    @property
    def Rows(self) -> int: ...
    @property
    def Cols(self) -> int: ...
    @property
    def TotalLength(self) -> int: ...
    @property
    def Data(self) -> list[list[float]]: ...
    @Data.setter
    def Data(self, value: list[list[float]]) -> None: ...
    def GetValueAt(self, Row: int, Col: int) -> float: ...
    def ReadData(self, Size: int) -> tuple[list[float]]: ...
    def SetValueAt(self, Row: int, Col: int, Value: float) -> None: ...
    def WriteData(self, Size: int, Data: list[float]) -> None: ...

class IMetadata:
    @property
    def NumberOfKeys(self) -> int: ...
    def ConvertFromBinary(self, data: list[int]) -> str: ...
    def ConvertToBinary(self, s: str) -> list[int]: ...
    def CreateGuid(self) -> str: ...
    def GetData(self, key: str) -> str: ...
    def GetKeyName(self, keyNumber: int) -> str: ...
    def RemoveData(self, key: str) -> bool: ...
    def SetData(self, key: str, value: str) -> None: ...

class ISettingsData:
    @property
    def NumberOfSettings(self) -> int: ...
    def GetBooleanArray(self, key: str, Size: int) -> tuple[bool, list[bool]]: ...
    def GetBooleanValue(self, key: str) -> tuple[bool, bool]: ...
    def GetByteArray(self, key: str, Size: int) -> tuple[bool, list[int]]: ...
    def GetByteValue(self, key: str) -> tuple[bool, int]: ...
    def GetDoubleArray(self, key: str, Size: int) -> tuple[bool, list[float]]: ...
    def GetDoubleValue(self, key: str) -> tuple[bool, float]: ...
    def GetEntryType(self, key: str) -> tuple[bool, SettingsDataType, bool, int]: ...
    def GetFloatArray(self, key: str, Size: int) -> tuple[bool, list[float]]: ...
    def GetFloatValue(self, key: str) -> tuple[bool, float]: ...
    def GetIntegerArray(self, key: str, Size: int) -> tuple[bool, list[int]]: ...
    def GetIntegerValue(self, key: str) -> tuple[bool, int]: ...
    def GetKeys(self) -> list[str]: ...
    def GetStringValue(self, key: str) -> tuple[bool, str]: ...
    def SetBooleanArray(self, key: str, Size: int, val: list[bool]) -> None: ...
    def SetBooleanValue(self, key: str, val: bool) -> None: ...
    def SetByteArray(self, key: str, Size: int, val: list[int]) -> None: ...
    def SetByteValue(self, key: str, val: int) -> None: ...
    def SetDoubleArray(self, key: str, Size: int, val: list[float]) -> None: ...
    def SetDoubleValue(self, key: str, val: float) -> None: ...
    def SetFloatArray(self, key: str, Size: int, val: list[float]) -> None: ...
    def SetFloatValue(self, key: str, val: float) -> None: ...
    def SetIntegerArray(self, key: str, Size: int, val: list[int]) -> None: ...
    def SetIntegerValue(self, key: str, val: int) -> None: ...
    def SetStringValue(self, key: str, val: str) -> None: ...

class IVectorData:
    @property
    def IsReadOnly(self) -> bool: ...
    @property
    def Length(self) -> int: ...
    @property
    def Data(self) -> list[float]: ...
    @Data.setter
    def Data(self, value: list[float]) -> None: ...
    def GetValueAt(self, position: int) -> float: ...
    def ReadData(self, Size: int) -> tuple[list[float]]: ...
    def SetValueAt(self, position: int, Value: float) -> None: ...
    def WriteData(self, Size: int, Data: list[float]) -> None: ...

class SettingsDataType:
    # None = 0
    Integer = 1
    Double = 2
    Float = 3
    String = 4
    Byte = 5
    Boolean = 6

class ZemaxColor:
    Default = 0
    Color1 = 1
    Color2 = 2
    Color3 = 3
    Color4 = 4
    Color5 = 5
    Color6 = 6
    Color7 = 7
    Color8 = 8
    Color9 = 9
    Color10 = 10
    Color11 = 11
    Color12 = 12
    Color13 = 13
    Color14 = 14
    Color15 = 15
    Color16 = 16
    Color17 = 17
    Color18 = 18
    Color19 = 19
    Color20 = 20
    Color21 = 21
    Color22 = 22
    Color23 = 23
    Color24 = 24
    NoColor = 25

class ZemaxOpacity:
    P100 = 0
    P90 = 1
    P80 = 2
    P70 = 3
    P60 = 4
    P50 = 5
    P40 = 6
    P30 = 7
    P20 = 8
    P10 = 9
    P00 = 10



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import IMessage
from zospy.api._ZOSAPI.Editors import ISolveData, SolveStatus

__all__ = (
    "FieldColumn",
    "FieldNormalizationType",
    "FieldPattern",
    "FieldType",
    "FNumberComputationType",
    "HuygensIntegralSettings",
    "IField",
    "IFields",
    "ISDAdvancedData",
    "ISDApertureData",
    "ISDEnvironmentData",
    "ISDFiles",
    "ISDMaterialCatalogData",
    "ISDNamedFilters",
    "ISDNonSeqData",
    "ISDPolarizationData",
    "ISDRayAimingData",
    "ISDTitleNotes",
    "ISDUnitsData",
    "ISurfaceSelection",
    "ISystemData",
    "IWavelength",
    "IWavelengths",
    "ParaxialRaysSetting",
    "PolarizationMethod",
    "QuadratureSteps",
    "RayAimingMethod",
    "RayAimingType",
    "ReferenceOPDSetting",
    "WavelengthPreset",
    "ZemaxAfocalModeUnits",
    "ZemaxAnalysisUnits",
    "ZemaxApertureType",
    "ZemaxApodizationType",
    "ZemaxMTFUnits",
    "ZemaxSourceUnits",
    "ZemaxSystemUnits",
    "ZemaxUnitPrefix",
)

class FieldColumn:
    Comment = 0
    X = 1
    Y = 2
    Weight = 3
    VDX = 4
    VDY = 5
    VCX = 6
    VCY = 7
    TAN = 8
    VAN = 8

class FieldNormalizationType:
    Radial = 0
    Rectangular = 1

class FieldPattern:
    UniformY = 0
    EqualAreaY = 1
    UniformX = 2
    EqualAreaX = 3
    Grid = 4
    UniformRadial = 5
    EqualAreaRadial = 6

class FieldType:
    Angle = 0
    ObjectHeight = 1
    ParaxialImageHeight = 2
    RealImageHeight = 3
    TheodoliteAngle = 4

class FNumberComputationType:
    TracingRays = 0
    PupilSizePosition = 1

class HuygensIntegralSettings:
    Auto = 0
    Planar = 1
    Spherical = 2

class IField:
    @property
    def IsActive(self) -> bool: ...
    @property
    def FieldNumber(self) -> int: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> None: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> None: ...
    @property
    def Weight(self) -> float: ...
    @Weight.setter
    def Weight(self, value: float) -> None: ...
    @property
    def VDX(self) -> float: ...
    @VDX.setter
    def VDX(self, value: float) -> None: ...
    @property
    def VDY(self) -> float: ...
    @VDY.setter
    def VDY(self, value: float) -> None: ...
    @property
    def VCX(self) -> float: ...
    @VCX.setter
    def VCX(self, value: float) -> None: ...
    @property
    def VCY(self) -> float: ...
    @VCY.setter
    def VCY(self, value: float) -> None: ...
    @property
    def VAN(self) -> float: ...
    @VAN.setter
    def VAN(self, value: float) -> None: ...
    @property
    def TAN(self) -> float: ...
    @TAN.setter
    def TAN(self, value: float) -> None: ...
    @property
    def Comment(self) -> str: ...
    @Comment.setter
    def Comment(self, value: str) -> None: ...
    @property
    def XSolve(self) -> SolveType: ...
    @property
    def YSolve(self) -> SolveType: ...
    @property
    def Ignore(self) -> bool: ...
    @Ignore.setter
    def Ignore(self, value: bool) -> None: ...
    def GetSolveData(self, forColumn: FieldColumn) -> ISolveData: ...
    def GetXSolveData(self) -> ISolveData: ...
    def GetYSolveData(self) -> ISolveData: ...
    def SetFixed(self, forColumn: FieldColumn) -> SolveStatus: ...
    def SetPickup(
        self, forColumn: FieldColumn, fromField: int, fromColumn: FieldColumn, scale: float, offset: float
    ) -> SolveStatus: ...
    def SetXFixed(self) -> SolveStatus: ...
    def SetXPickup(self, fromField: int, fromColumn: FieldColumn, scale: float, offset: float) -> SolveStatus: ...
    def SetYFixed(self) -> SolveStatus: ...
    def SetYPickup(self, fromField: int, fromColumn: FieldColumn, scale: float, offset: float) -> SolveStatus: ...

class IFields:
    @property
    def NumberOfFields(self) -> int: ...
    @property
    def Normalization(self) -> FieldNormalizationType: ...
    @Normalization.setter
    def Normalization(self, value: FieldNormalizationType) -> None: ...
    def AddField(self, X: float, Y: float, Weight: float) -> IField: ...
    def ApplyFieldWizard(
        self,
        pattern: FieldPattern,
        numberOfYFields: int,
        maxFieldY: float,
        numberOfXFields: int,
        maxFieldX: float,
        startAt: int,
        overwrite: bool,
        includePickups: bool,
    ) -> IMessage: ...
    def ClearVignetting(self) -> None: ...
    def ConvertToFieldType(self, type: FieldType) -> IMessage: ...
    def DeleteAllFields(self) -> int: ...
    def DeleteFieldAt(self, fieldNumber: int) -> bool: ...
    def DeleteFieldsAt(self, fieldNumber: int, numberOfFields: int) -> int: ...
    def GetField(self, position: int) -> IField: ...
    def GetFieldType(self) -> FieldType: ...
    def InsertFieldAt(self, fieldNumber: int) -> IField: ...
    def RemoveField(self, position: int) -> bool: ...
    def SetFieldType(self, type: FieldType) -> None: ...
    def SetVignetting(self) -> None: ...

class ISDAdvancedData:
    @property
    def ReferenceOPD(self) -> ReferenceOPDSetting: ...
    @ReferenceOPD.setter
    def ReferenceOPD(self, value: ReferenceOPDSetting) -> None: ...
    @property
    def ParaxialRays(self) -> ParaxialRaysSetting: ...
    @ParaxialRays.setter
    def ParaxialRays(self, value: ParaxialRaysSetting) -> None: ...
    @property
    def FNumMethod(self) -> FNumberComputationType: ...
    @FNumMethod.setter
    def FNumMethod(self, value: FNumberComputationType) -> None: ...
    @property
    def DontPrintCoordinateBreakData(self) -> bool: ...
    @DontPrintCoordinateBreakData.setter
    def DontPrintCoordinateBreakData(self, value: bool) -> None: ...
    @property
    def TurnOffThreading(self) -> bool: ...
    @TurnOffThreading.setter
    def TurnOffThreading(self, value: bool) -> None: ...
    @property
    def OPDModulo2PI(self) -> bool: ...
    @OPDModulo2PI.setter
    def OPDModulo2PI(self, value: bool) -> None: ...
    @property
    def IncludeCalculatedDataInSessionFile(self) -> bool: ...
    @IncludeCalculatedDataInSessionFile.setter
    def IncludeCalculatedDataInSessionFile(self, value: bool) -> None: ...
    @property
    def IncludeToleranceDataInSessionFile(self) -> bool: ...
    @IncludeToleranceDataInSessionFile.setter
    def IncludeToleranceDataInSessionFile(self, value: bool) -> None: ...
    @property
    def HuygensIntegralMethod(self) -> HuygensIntegralSettings: ...
    @HuygensIntegralMethod.setter
    def HuygensIntegralMethod(self, value: HuygensIntegralSettings) -> None: ...

class ISDApertureData:
    @property
    def ApertureType(self) -> ZemaxApertureType: ...
    @ApertureType.setter
    def ApertureType(self, value: ZemaxApertureType) -> None: ...
    @property
    def ApertureValue(self) -> float: ...
    @ApertureValue.setter
    def ApertureValue(self, value: float) -> None: ...
    @property
    def ApodizationType(self) -> ZemaxApodizationType: ...
    @ApodizationType.setter
    def ApodizationType(self, value: ZemaxApodizationType) -> None: ...
    @property
    def ApodizationFactor(self) -> float: ...
    @ApodizationFactor.setter
    def ApodizationFactor(self, value: float) -> None: ...
    @property
    def ApodizationFactorIsUsed(self) -> bool: ...
    @property
    def SemiDiameterMargin(self) -> float: ...
    @SemiDiameterMargin.setter
    def SemiDiameterMargin(self, value: float) -> None: ...
    @property
    def SemiDiameterMarginPct(self) -> float: ...
    @SemiDiameterMarginPct.setter
    def SemiDiameterMarginPct(self, value: float) -> None: ...
    @property
    def GCRS(self) -> ISurfaceSelection: ...
    @property
    def TelecentricObjectSpace(self) -> bool: ...
    @TelecentricObjectSpace.setter
    def TelecentricObjectSpace(self, value: bool) -> None: ...
    @property
    def AFocalImageSpace(self) -> bool: ...
    @AFocalImageSpace.setter
    def AFocalImageSpace(self, value: bool) -> None: ...
    @property
    def IterateSolvesWhenUpdating(self) -> bool: ...
    @IterateSolvesWhenUpdating.setter
    def IterateSolvesWhenUpdating(self, value: bool) -> None: ...
    @property
    def FastSemiDiameters(self) -> bool: ...
    @FastSemiDiameters.setter
    def FastSemiDiameters(self, value: bool) -> None: ...
    @property
    def CheckGRINApertures(self) -> bool: ...
    @CheckGRINApertures.setter
    def CheckGRINApertures(self, value: bool) -> None: ...

class ISDEnvironmentData:
    @property
    def AdjustIndexToEnvironment(self) -> bool: ...
    @AdjustIndexToEnvironment.setter
    def AdjustIndexToEnvironment(self, value: bool) -> None: ...
    @property
    def Temperature(self) -> float: ...
    @Temperature.setter
    def Temperature(self, value: float) -> None: ...
    @property
    def Pressure(self) -> float: ...
    @Pressure.setter
    def Pressure(self, value: float) -> None: ...

class ISDFiles:
    @property
    def CoatingFile(self) -> str: ...
    @CoatingFile.setter
    def CoatingFile(self, value: str) -> None: ...
    @property
    def ScatterProfile(self) -> str: ...
    @ScatterProfile.setter
    def ScatterProfile(self, value: str) -> None: ...
    @property
    def ABgDataFile(self) -> str: ...
    @ABgDataFile.setter
    def ABgDataFile(self, value: str) -> None: ...
    @property
    def GradiumProfile(self) -> str: ...
    @GradiumProfile.setter
    def GradiumProfile(self, value: str) -> None: ...
    def GetABgDataFiles(self) -> list[str]: ...
    def GetCoatingFiles(self) -> list[str]: ...
    def GetGradiumProfiles(self) -> list[str]: ...
    def GetScatterProfiles(self) -> list[str]: ...
    def ReloadFiles(self) -> None: ...

class ISDMaterialCatalogData:
    def AddCatalog(self, catalog: str) -> bool: ...
    def GetAvailableCatalogs(self) -> list[str]: ...
    def GetCatalogsInUse(self) -> list[str]: ...
    def GetMaterialsInCatalog(self, catalog: str) -> list[str]: ...
    def IsCatalogInUse(self, catalog: str) -> bool: ...
    def RemoveCatalog(self, catalog: str) -> bool: ...

class ISDNamedFilters:
    @property
    def NamedFilters(self) -> str: ...
    @NamedFilters.setter
    def NamedFilters(self, value: str) -> None: ...

class ISDNonSeqData:
    @property
    def MaximumIntersectionsPerRay(self) -> int: ...
    @MaximumIntersectionsPerRay.setter
    def MaximumIntersectionsPerRay(self, value: int) -> None: ...
    @property
    def MaximumSegmentsPerRay(self) -> int: ...
    @MaximumSegmentsPerRay.setter
    def MaximumSegmentsPerRay(self, value: int) -> None: ...
    @property
    def MaximumNestedTouchingObjects(self) -> int: ...
    @MaximumNestedTouchingObjects.setter
    def MaximumNestedTouchingObjects(self, value: int) -> None: ...
    @property
    def MaximumSourceFileRaysInMemory(self) -> int: ...
    @MaximumSourceFileRaysInMemory.setter
    def MaximumSourceFileRaysInMemory(self, value: int) -> None: ...
    @property
    def MinimumRelativeRayIntensity(self) -> float: ...
    @MinimumRelativeRayIntensity.setter
    def MinimumRelativeRayIntensity(self, value: float) -> None: ...
    @property
    def MinimumAbsoluteRayIntensity(self) -> float: ...
    @MinimumAbsoluteRayIntensity.setter
    def MinimumAbsoluteRayIntensity(self, value: float) -> None: ...
    @property
    def GlueDistanceInLensUnits(self) -> float: ...
    @GlueDistanceInLensUnits.setter
    def GlueDistanceInLensUnits(self, value: float) -> None: ...
    @property
    def MissedRayDrawDistanceInLensUnits(self) -> float: ...
    @MissedRayDrawDistanceInLensUnits.setter
    def MissedRayDrawDistanceInLensUnits(self, value: float) -> None: ...
    @property
    def SimpleRaySplitting(self) -> bool: ...
    @SimpleRaySplitting.setter
    def SimpleRaySplitting(self, value: bool) -> None: ...
    @property
    def RetraceSourceRaysUponFileOpen(self) -> bool: ...
    @RetraceSourceRaysUponFileOpen.setter
    def RetraceSourceRaysUponFileOpen(self, value: bool) -> None: ...

class ISDPolarizationData:
    @property
    def ConvertThinFilmPhaseToRayEquivalent(self) -> bool: ...
    @ConvertThinFilmPhaseToRayEquivalent.setter
    def ConvertThinFilmPhaseToRayEquivalent(self, value: bool) -> None: ...
    @property
    def Unpolarized(self) -> bool: ...
    @Unpolarized.setter
    def Unpolarized(self, value: bool) -> None: ...
    @property
    def Jx(self) -> float: ...
    @Jx.setter
    def Jx(self, value: float) -> None: ...
    @property
    def Jy(self) -> float: ...
    @Jy.setter
    def Jy(self, value: float) -> None: ...
    @property
    def XPhase(self) -> float: ...
    @XPhase.setter
    def XPhase(self, value: float) -> None: ...
    @property
    def YPhase(self) -> float: ...
    @YPhase.setter
    def YPhase(self, value: float) -> None: ...
    @property
    def Method(self) -> PolarizationMethod: ...
    @Method.setter
    def Method(self, value: PolarizationMethod) -> None: ...

class ISDRayAimingData:
    @property
    def RayAiming(self) -> RayAimingMethod: ...
    @RayAiming.setter
    def RayAiming(self, value: RayAimingMethod) -> None: ...
    @property
    def Method(self) -> RayAimingType: ...
    @Method.setter
    def Method(self, value: RayAimingType) -> None: ...
    @property
    def UseRayAimingCache(self) -> bool: ...
    @UseRayAimingCache.setter
    def UseRayAimingCache(self, value: bool) -> None: ...
    @property
    def UseRobustRayAiming(self) -> bool: ...
    @UseRobustRayAiming.setter
    def UseRobustRayAiming(self, value: bool) -> None: ...
    @property
    def ScalePupilShiftFactorsByField(self) -> bool: ...
    @ScalePupilShiftFactorsByField.setter
    def ScalePupilShiftFactorsByField(self, value: bool) -> None: ...
    @property
    def AutomaticallyCalculatePupilShiftsIsChecked(self) -> bool: ...
    @AutomaticallyCalculatePupilShiftsIsChecked.setter
    def AutomaticallyCalculatePupilShiftsIsChecked(self, value: bool) -> None: ...
    @property
    def UseEnhancedRayAiming(self) -> bool: ...
    @UseEnhancedRayAiming.setter
    def UseEnhancedRayAiming(self, value: bool) -> None: ...
    @property
    def UseAdvancedConvergence(self) -> bool: ...
    @UseAdvancedConvergence.setter
    def UseAdvancedConvergence(self, value: bool) -> None: ...
    @property
    def UseFallBackSearchDuringCacheSetup(self) -> bool: ...
    @UseFallBackSearchDuringCacheSetup.setter
    def UseFallBackSearchDuringCacheSetup(self, value: bool) -> None: ...
    @property
    def PupilShiftX(self) -> float: ...
    @PupilShiftX.setter
    def PupilShiftX(self, value: float) -> None: ...
    @property
    def PupilShiftY(self) -> float: ...
    @PupilShiftY.setter
    def PupilShiftY(self, value: float) -> None: ...
    @property
    def PupilShiftZ(self) -> float: ...
    @PupilShiftZ.setter
    def PupilShiftZ(self, value: float) -> None: ...
    @property
    def PupilCompressX(self) -> float: ...
    @PupilCompressX.setter
    def PupilCompressX(self, value: float) -> None: ...
    @property
    def PupilCompressY(self) -> float: ...
    @PupilCompressY.setter
    def PupilCompressY(self, value: float) -> None: ...
    @property
    def NumStepsCacheSetup(self) -> int: ...
    @NumStepsCacheSetup.setter
    def NumStepsCacheSetup(self, value: int) -> None: ...

class ISDTitleNotes:
    @property
    def Title(self) -> str: ...
    @Title.setter
    def Title(self, value: str) -> None: ...
    @property
    def Notes(self) -> str: ...
    @Notes.setter
    def Notes(self, value: str) -> None: ...
    @property
    def Author(self) -> str: ...
    @Author.setter
    def Author(self, value: str) -> None: ...

class ISDUnitsData:
    @property
    def LensUnits(self) -> ZemaxSystemUnits: ...
    @LensUnits.setter
    def LensUnits(self, value: ZemaxSystemUnits) -> None: ...
    @property
    def SourceUnitPrefix(self) -> ZemaxUnitPrefix: ...
    @SourceUnitPrefix.setter
    def SourceUnitPrefix(self, value: ZemaxUnitPrefix) -> None: ...
    @property
    def SourceUnits(self) -> ZemaxSourceUnits: ...
    @SourceUnits.setter
    def SourceUnits(self, value: ZemaxSourceUnits) -> None: ...
    @property
    def AnalysisUnitPrefix(self) -> ZemaxUnitPrefix: ...
    @AnalysisUnitPrefix.setter
    def AnalysisUnitPrefix(self, value: ZemaxUnitPrefix) -> None: ...
    @property
    def AnalysisUnits(self) -> ZemaxAnalysisUnits: ...
    @AnalysisUnits.setter
    def AnalysisUnits(self, value: ZemaxAnalysisUnits) -> None: ...
    @property
    def AfocalModeUnits(self) -> ZemaxAfocalModeUnits: ...
    @AfocalModeUnits.setter
    def AfocalModeUnits(self, value: ZemaxAfocalModeUnits) -> None: ...
    @property
    def MTFUnits(self) -> ZemaxMTFUnits: ...
    @MTFUnits.setter
    def MTFUnits(self, value: ZemaxMTFUnits) -> None: ...

class ISurfaceSelection:
    @property
    def FirstAllowedSurface(self) -> int: ...
    @property
    def LastAllowedSurface(self) -> int: ...
    @property
    def ImageSurface(self) -> int: ...
    @property
    def StopSurface(self) -> int: ...
    def GetSelectedSurface(self) -> int: ...
    def SetSelectedSurface(self, Surface: int) -> bool: ...
    def UseImageSurface(self) -> bool: ...
    def UseObjectSurface(self) -> bool: ...
    def UseStopSurface(self) -> bool: ...

class ISystemData:
    @property
    def Aperture(self) -> ISDApertureData: ...
    @property
    def Wavelengths(self) -> IWavelengths: ...
    @property
    def Fields(self) -> IFields: ...
    @property
    def Environment(self) -> ISDEnvironmentData: ...
    @property
    def Polarization(self) -> ISDPolarizationData: ...
    @property
    def Advanced(self) -> ISDAdvancedData: ...
    @property
    def RayAiming(self) -> ISDRayAimingData: ...
    @property
    def MaterialCatalogs(self) -> ISDMaterialCatalogData: ...
    @property
    def TitleNotes(self) -> ISDTitleNotes: ...
    @property
    def Files(self) -> ISDFiles: ...
    @property
    def Units(self) -> ISDUnitsData: ...
    @property
    def NonSequentialData(self) -> ISDNonSeqData: ...
    @property
    def NamedFiltersData(self) -> ISDNamedFilters: ...

class IWavelength:
    @property
    def WavelengthNumber(self) -> int: ...
    @property
    def IsActive(self) -> bool: ...
    @property
    def IsPrimary(self) -> bool: ...
    @property
    def Wavelength(self) -> float: ...
    @Wavelength.setter
    def Wavelength(self, value: float) -> None: ...
    @property
    def Weight(self) -> float: ...
    @Weight.setter
    def Weight(self, value: float) -> None: ...
    def MakePrimary(self) -> None: ...

class IWavelengths:
    @property
    def NumberOfWavelengths(self) -> int: ...
    def AddWavelength(self, Wavelength: float, Weight: float) -> IWavelength: ...
    def GaussianQuadrature(self, minWave: float, maxWave: float, numSteps: QuadratureSteps) -> bool: ...
    def GetWavelength(self, position: int) -> IWavelength: ...
    def RemoveWavelength(self, position: int) -> bool: ...
    def SelectWavelengthPreset(self, preset: WavelengthPreset) -> bool: ...

class ParaxialRaysSetting:
    ConsiderCoordinateBreaks = 0
    IgnoreCoordinateBreaks = 1

class PolarizationMethod:
    XAxisMethod = 0
    YAxisMethod = 1
    ZAxisMethod = 2

class QuadratureSteps:
    S2 = 0
    S4 = 1
    S6 = 2
    S8 = 3
    S10 = 4
    S12 = 5

class RayAimingMethod:
    Off = 0
    Paraxial = 1
    Real = 2

class RayAimingType:
    Heuristic = 0
    Optimize = 1

class ReferenceOPDSetting:
    Absolute = 0
    Infinity = 1
    ExitPupil = 2
    Absolute2 = 3

class WavelengthPreset:
    FdC_Visible = 0
    Photopic_Bright = 1
    Scotopic_Dark = 2
    HeNe_0p6328 = 3
    HeNe_0p5438 = 4
    Argon_0p4880 = 5
    Argon_0p5145 = 6
    NDYAG_1p0641 = 7
    NDGlass_1p054 = 8
    CO2_10p60 = 9
    CrLiSAF_0p840 = 10
    TiAl203_0p760 = 11
    Ruby_0p6943 = 12
    HeCadmium_0p4416 = 13
    HeCadmium_0p3536 = 14
    HeCadmium_0p3250 = 15
    t_1p014 = 16
    r_0p707 = 17
    C_0p656 = 18
    d_0p587 = 19
    F_0p486 = 20
    g_0p436 = 21
    i_0p365 = 22
    Fp_0p365 = 23
    e_0p54607 = 24
    Cp_0p6438469 = 25
    FpeCp_Visible = 26
    THz_193p10 = 27

class ZemaxAfocalModeUnits:
    Microradians = 0
    Milliradians = 1
    Radians = 2
    ArcSeconds = 3
    ArcMinutes = 4
    Degrees = 5

class ZemaxAnalysisUnits:
    WattsPerMMSq = 0
    WattsPerCMSq = 1
    WattsPerinSq = 2
    WattsPerMSq = 3
    WattsPerftSq = 4

class ZemaxApertureType:
    EntrancePupilDiameter = 0
    ImageSpaceFNum = 1
    ObjectSpaceNA = 2
    FloatByStopSize = 3
    ParaxialWorkingFNum = 4
    ObjectConeAngle = 5

class ZemaxApodizationType:
    Uniform = 0
    Gaussian = 1
    CosineCubed = 2

class ZemaxMTFUnits:
    CyclesPerMillimeter = 0
    CyclesPerMilliradian = 1

class ZemaxSourceUnits:
    Watts = 0
    Lumens = 1
    Joules = 2

class ZemaxSystemUnits:
    Millimeters = 0
    Centimeters = 1
    Inches = 2
    Meters = 3

class ZemaxUnitPrefix:
    Femto = 0
    Pico = 1
    Nano = 2
    Micro = 3
    Milli = 4
    # None = 5
    Kilo = 6
    Mega = 7
    Giga = 8
    Tera = 9



"""Wavefront Map analysis."""

from __future__ import annotations

from typing import Annotated, Literal, Union

import pandas as pd
from pandas import DataFrame
from pydantic import Field

from zospy.analyses.base import BaseAnalysisWrapper
from zospy.analyses.decorators import analysis_settings
from zospy.analyses.parsers.types import ZOSAPIConstant  # noqa: TCH001
from zospy.api import constants
from zospy.utils.zputils import standardize_sampling

__all__ = ("WavefrontMap", "WavefrontMapSettings")


@analysis_settings
class WavefrontMapSettings:
    """Settings for the Wavefront Map analysis.

    Attributes
    ----------
    field : int
        The field that is to be analyzed. Defaults to 1.
    surface  str | int
        The surface that is to be analyzed. Either 'Image', 'Object' or an integer. Defaults to 'Image'.
    wavelength : int
        The wavelength number to use for the analysis. Defaults to 1.
    show_as : constants.Analysis.ShowAs | str
        Defines the output plot format. Defaults to 'Surface'.
    rotation : constants.Analysis.Settings.Rotations | str
        The rotation or surface plots for viewing. Defaults to 'Rotate_0'.
    sampling : str
        The sampling. Defaults to '64x64'.
    polarization : constants.Analysis.Settings.Polarizations | str | None
        The polarization that is accounted for. When set to None, polarization is ignored. Defaults to None.
    reference_to_primary : bool
        Defines whether the aberrations are referenced to the reference sphere for the used wavelength or for the
        primary wavelength. If True, the reference sphere for the primary wavelength is used. Defaults to False.
    use_exit_pupil : bool
        Defines whether the exit pupil shape is used. Defaults to True.
    remove_tilt : bool
        Defines whether linear x- and y-tilt is removed from the data. Defaults to False.
    scale : float
        The scale factor for surface plots. Defaults to False.
    sub_aperture_x : float
        The sub-aperture x coordinate. Defaults to 0.0
    sub_aperture_y : float
        The sub-aperture y coordinate. Defaults to 0.0
    sub_aperture_r : float
        The sub-aperture radius. Defaults to 1.0
    contour_format : str
        The contour format. Only used when show-As is set to 'Contour'. If set to an empty string, OpticStdio ignores
        it. Defaults to ''.
    """

    field: int = Field(default=1, ge=1, description="Field number")
    surface: Literal["Image"] | Annotated[int, Field(ge=1)] = Field(default="Image", description="Surface number")
    wavelength: int = Field(default=1, ge=1, description="Wavelength number")
    show_as: ZOSAPIConstant("Analysis.ShowAs") = Field(default="Surface", description="Show as")
    rotation: ZOSAPIConstant("Analysis.Settings.Rotations") = Field(default="Rotate_0", description="Rotation")
    sampling: str = Field(default="64x64", description="Sampling grid")
    polarization: ZOSAPIConstant("Analysis.Settings.Polarizations") | None = Field(
        default=None, description="Polarization"
    )
    reference_to_primary: bool = Field(default=False, description="Reference to primary wavelength")
    use_exit_pupil: bool = Field(default=True, description="Use exit pupil shape")
    remove_tilt: bool = Field(default=False, description="Remove linear tilt from the data")
    scale: float = Field(default=1.0, description="Scale factor for the surface plots")
    sub_aperture_x: float = Field(default=0.0, description="Sub aperture X coordinate")
    sub_aperture_y: float = Field(default=0.0, description="Sub aperture Y coordinate")
    sub_aperture_r: float = Field(default=1.0, description="Sub aperture radius")
    contour_format: str = Field(default="", description="Contour format")


class WavefrontMap(BaseAnalysisWrapper[Union[DataFrame, None], WavefrontMapSettings], analysis_type="WavefrontMap"):
    """Wavefront Map analysis.

    Warnings
    --------
    The ZOS-API returns a datagrid with an empty first row and first column. Given normalized within the wavefront
    map, the datagrid should span from x=-1 to x=1, and y=-1 to y=1. The provided datagrid.MinX and .MinY indeed
    point to (-1, -1), but the provided width the datagrid cells make the width of the entire datagrid 2 +
    1*cell_width. The same holds for the height. Thus, ZOSPy drops the empty first row and column, resulting in a
    centered wavefront map ranging from -1 to 1 in both x and y.
    """

    def __init__(
        self,
        *,
        field: int = 1,
        surface: Literal["Image"] | int = "Image",
        wavelength: int = 1,
        show_as: constants.Analysis.ShowAs | str = "Surface",
        rotation: constants.Analysis.Settings.Rotations | str = "Rotate_0",
        sampling: constants.Analysis.SampleSizes | str = "64x64",
        polarization: constants.Analysis.Settings.Polarizations | str | None = None,
        reference_to_primary: bool = False,
        use_exit_pupil: bool = True,
        remove_tilt: bool = False,
        scale: float = 1.0,
        sub_aperture_x: float = 0.0,
        sub_aperture_y: float = 0.0,
        sub_aperture_r: float = 1.0,
        contour_format: str = "",
    ):
        super().__init__(settings_kws=locals())

    def run_analysis(self) -> DataFrame:
        """Run the Wavefront Map analysis.

        See Also
        --------
        WavefrontMapSettings : Settings for the Wavefront Map analysis.
        """
        self.analysis.set_field(self.settings.field)
        self.analysis.set_surface(self.settings.surface)
        self.analysis.set_wavelength(self.settings.wavelength)
        self.analysis.Settings.ShowAs = constants.process_constant(constants.Analysis.ShowAs, self.settings.show_as)
        self.analysis.Settings.Rotation = constants.process_constant(
            constants.Analysis.Settings.Rotations, self.settings.rotation
        )
        self.analysis.Settings.Sampling = constants.process_constant(
            constants.Analysis.SampleSizes, standardize_sampling(self.settings.sampling)
        )
        self.analysis.Settings.Polarization = constants.process_constant(
            constants.Analysis.Settings.Polarizations, self.settings.polarization
        )
        self.analysis.Settings.ReferenceToPrimary = self.settings.reference_to_primary
        self.analysis.Settings.UseExitPupil = self.settings.use_exit_pupil
        self.analysis.Settings.RemoveTilt = self.settings.remove_tilt
        self.analysis.Settings.Scale = self.settings.scale
        self.analysis.Settings.Subaperture_X = self.settings.sub_aperture_x
        self.analysis.Settings.Subaperture_Y = self.settings.sub_aperture_y
        self.analysis.Settings.Subaperture_R = self.settings.sub_aperture_r
        self.analysis.Settings.ContourFormat = self.settings.contour_format

        self.analysis.ApplyAndWaitForCompletion()

        datagrid = self.get_data_grid(cell_origin="bottom_left")
        return pd.DataFrame(datagrid.values[1:, 1:], columns=datagrid.columns[:-1], index=datagrid.index[:-1])



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Tools import ISystemTool

__all__ = ("ConvertAsphereTypes", "FindBestAsphereTypes", "IConvertAsphereType", "IFindBestAsphere")

class ConvertAsphereTypes:
    EvenAsphere = 0
    ExtendedAsphere = 1
    QTypeAsphereType0 = 2
    QTypeAsphereType1 = 3

class FindBestAsphereTypes:
    Conic = 0
    Polynomial_4thOrder = 1
    Polynomial_6thOrder = 2
    Polynomial_8thOrder = 3
    Polynomial_10thOrder = 4
    Polynomial_12thOrder = 5

class IConvertAsphereType(ISystemTool):
    @property
    def Surface(self) -> int: ...
    @Surface.setter
    def Surface(self, value: int) -> None: ...
    @property
    def AsphereType(self) -> ConvertAsphereTypes: ...
    @AsphereType.setter
    def AsphereType(self, value: ConvertAsphereTypes) -> None: ...
    @property
    def NumberOfTerms(self) -> int: ...
    @NumberOfTerms.setter
    def NumberOfTerms(self, value: int) -> None: ...
    @property
    def ExcludeSecondOrderTerm(self) -> bool: ...
    @ExcludeSecondOrderTerm.setter
    def ExcludeSecondOrderTerm(self, value: bool) -> None: ...
    @property
    def AllowRadiusVariation(self) -> bool: ...
    @AllowRadiusVariation.setter
    def AllowRadiusVariation(self, value: bool) -> None: ...
    @property
    def AllowConicVariation(self) -> bool: ...
    @AllowConicVariation.setter
    def AllowConicVariation(self, value: bool) -> None: ...
    @property
    def ResidualRMSError(self) -> float: ...
    @property
    def IsAutomatic(self) -> None: ...
    @IsAutomatic.setter
    def IsAutomatic(self, value: bool) -> None: ...

class IFindBestAsphere(ISystemTool):
    @property
    def FirstSurface(self) -> int: ...
    @FirstSurface.setter
    def FirstSurface(self, value: int) -> None: ...
    @property
    def LastSurface(self) -> int: ...
    @LastSurface.setter
    def LastSurface(self, value: int) -> None: ...
    @property
    def AsphereType(self) -> FindBestAsphereTypes: ...
    @AsphereType.setter
    def AsphereType(self, value: FindBestAsphereTypes) -> None: ...
    @property
    def StartMeritFunction(self) -> float: ...
    @property
    def BestMeritFunction(self) -> float: ...
    @property
    def BestSurface(self) -> int: ...



from pathlib import Path

import pytest

from zospy.analyses.parsers import base

available_grammars = [
    g.stem for g in Path(base.__file__).parent.joinpath("grammars").glob("*.lark") if g.stem != "zospy"
]


@pytest.mark.parametrize("grammar", available_grammars)
def test_load_grammar(grammar: str):
    grammar = base.load_grammar(grammar)
    assert grammar is not None


def test_load_nonexistent_grammar():
    with pytest.raises(FileNotFoundError, match="Grammar file nonexistent.lark not found"):
        base.load_grammar("nonexistent")



"""Parsers for OpticStudio analysis text output files."""

from zospy.analyses.parsers import types
from zospy.analyses.parsers.base import load_grammar, parse
from zospy.analyses.parsers.transformers import ZospyTransformer

__all__ = ("load_grammar", "parse", "ZospyTransformer", "types")



import numpy as np
import pytest

from zospy.analyses.old.raysandspots import ray_fan, single_ray_trace
from zospy.api.config import DECIMAL_POINT

pytestmark = pytest.mark.old_analyses


class TestSingleRayTrace:
    def test_can_run_single_ray_trace(self, simple_system):
        result = single_ray_trace(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = single_ray_trace(simple_system)

        assert result.from_json(result.to_json())

    @pytest.mark.parametrize(
        "hx,hy,px,py,raytrace_type,global_coordinates",
        [
            (1, 1, 0, 0, "DirectionCosines", False),
            (0, 0, 1, 1, "DirectionCosines", True),
            (1, 1, 0, 0, "TangentAngle", True),
        ],
    )
    def test_single_ray_trace_returns_correct_result(
        self, simple_system, hx, hy, px, py, raytrace_type, global_coordinates, expected_data
    ):
        result = single_ray_trace(
            simple_system,
            hx=hx,
            hy=hy,
            px=px,
            py=py,
            raytrace_type=raytrace_type,
            global_coordinates=global_coordinates,
        )

        assert np.allclose(
            result.Data.ParaxialRayTraceData.select_dtypes(float),
            expected_data.Data.ParaxialRayTraceData.select_dtypes(float),
            equal_nan=True,
        )
        assert np.allclose(
            result.Data.RealRayTraceData.select_dtypes(float),
            expected_data.Data.RealRayTraceData.select_dtypes(float),
            equal_nan=True,
        )

    @pytest.mark.parametrize(
        "hx,hy,px,py,raytrace_type,global_coordinates",
        [
            (1, 1, 0, 0, "DirectionCosines", False),
            (0, 0, 1, 1, "DirectionCosines", True),
            (1, 1, 0, 0, "TangentAngle", True),
        ],
    )
    def test_single_ray_trace_matches_reference_data(
        self, simple_system, hx, hy, px, py, raytrace_type, global_coordinates, reference_data
    ):
        result = single_ray_trace(
            simple_system,
            hx=hx,
            hy=hy,
            px=px,
            py=py,
            raytrace_type=raytrace_type,
            global_coordinates=global_coordinates,
        )

        assert np.allclose(
            result.Data.ParaxialRayTraceData.select_dtypes(float),
            reference_data.Data.ParaxialRayTraceData.select_dtypes(float),
            equal_nan=True,
        )
        assert np.allclose(
            result.Data.RealRayTraceData.select_dtypes(float),
            reference_data.Data.RealRayTraceData.select_dtypes(float),
            equal_nan=True,
        )


class TestRayFan:
    def test_can_run_ray_fan(self, simple_system):
        result = ray_fan(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = ray_fan(simple_system)

        assert result.from_json(result.to_json())

    @pytest.mark.parametrize(
        "plot_scale,number_of_rays,tangential,sagittal",
        [
            (0, 20, "Aberration_Y", "Aberration_X"),
            (1, 40, "Aberration_Y", "Aberration_X"),
            (0, 20, "Aberration_X", "Aberration_Y"),
        ],
    )
    def test_ray_fan_returns_correct_result(
        self, simple_system, plot_scale, number_of_rays, tangential, sagittal, expected_data
    ):
        result = ray_fan(
            simple_system,
            plot_scale=plot_scale,
            number_of_rays=number_of_rays,
            tangential=tangential,
            sagittal=sagittal,
        )

        assert np.allclose(
            result.Data[f"Tangential fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
            expected_data.Data[f"Tangential fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
        )
        assert np.allclose(
            result.Data[f"Sagittal fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
            expected_data.Data[f"Sagittal fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
        )

    @pytest.mark.parametrize(
        "plot_scale,number_of_rays,tangential,sagittal",
        [
            (0, 20, "Aberration_Y", "Aberration_X"),
            (1, 40, "Aberration_Y", "Aberration_X"),
            (0, 20, "Aberration_X", "Aberration_Y"),
        ],
    )
    def test_ray_fan_matches_reference_data(
        self, simple_system, plot_scale, number_of_rays, tangential, sagittal, reference_data
    ):
        result = ray_fan(
            simple_system,
            plot_scale=plot_scale,
            number_of_rays=number_of_rays,
            tangential=tangential,
            sagittal=sagittal,
        )

        assert np.allclose(
            result.Data[f"Tangential fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
            reference_data.Data[f"Tangential fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
        )
        assert np.allclose(
            result.Data[f"Sagittal fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
            reference_data.Data[f"Sagittal fan, field number 1 = 0{DECIMAL_POINT}0000 (deg)"],
        )



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("MeritColumn", "MeritOperandType")

class MeritColumn:
    Comment: MeritColumn = None
    Param1: MeritColumn = None
    Param2: MeritColumn = None
    Param3: MeritColumn = None
    Param4: MeritColumn = None
    Param5: MeritColumn = None
    Param6: MeritColumn = None
    Param7: MeritColumn = None
    Param8: MeritColumn = None
    Target: MeritColumn = None
    Weight: MeritColumn = None
    Value: MeritColumn = None
    Contrib: MeritColumn = None

class MeritOperandType:
    ACOS: MeritOperandType = None
    ABSO: MeritOperandType = None
    CVVA: MeritOperandType = None
    DENC: MeritOperandType = None
    DENF: MeritOperandType = None
    DIFF: MeritOperandType = None
    AMAG: MeritOperandType = None
    ANAC: MeritOperandType = None
    ANAR: MeritOperandType = None
    ANAX: MeritOperandType = None
    ANAY: MeritOperandType = None
    ANCX: MeritOperandType = None
    ANCY: MeritOperandType = None
    ASIN: MeritOperandType = None
    ASTI: MeritOperandType = None
    ATAN: MeritOperandType = None
    AXCL: MeritOperandType = None
    BIOC: MeritOperandType = None
    CTVA: MeritOperandType = None
    CVGT: MeritOperandType = None
    CVLT: MeritOperandType = None
    CVOL: MeritOperandType = None
    DIMX: MeritOperandType = None
    DISC: MeritOperandType = None
    DISG: MeritOperandType = None
    DIST: MeritOperandType = None
    DIVI: MeritOperandType = None
    DLTN: MeritOperandType = None
    DMFS: MeritOperandType = None
    DMGT: MeritOperandType = None
    DMLT: MeritOperandType = None
    DMVA: MeritOperandType = None
    DXDX: MeritOperandType = None
    DXDY: MeritOperandType = None
    DYDX: MeritOperandType = None
    DYDY: MeritOperandType = None
    EFFL: MeritOperandType = None
    EFLX: MeritOperandType = None
    EFLY: MeritOperandType = None
    ENDX: MeritOperandType = None
    ENPP: MeritOperandType = None
    EPDI: MeritOperandType = None
    EQUA: MeritOperandType = None
    ETGT: MeritOperandType = None
    ETLT: MeritOperandType = None
    ETVA: MeritOperandType = None
    EXPP: MeritOperandType = None
    FCGS: MeritOperandType = None
    FCGT: MeritOperandType = None
    FCUR: MeritOperandType = None
    FICL: MeritOperandType = None
    FICP: MeritOperandType = None
    FOUC: MeritOperandType = None
    GBPD: MeritOperandType = None
    GBPR: MeritOperandType = None
    GBPS: MeritOperandType = None
    GBPW: MeritOperandType = None
    GBPP: MeritOperandType = None
    GBSD: MeritOperandType = None
    P1GT: MeritOperandType = None
    P1LT: MeritOperandType = None
    P1VA: MeritOperandType = None
    P2GT: MeritOperandType = None
    P2LT: MeritOperandType = None
    P2VA: MeritOperandType = None
    P3GT: MeritOperandType = None
    P3LT: MeritOperandType = None
    P3VA: MeritOperandType = None
    P4GT: MeritOperandType = None
    P4LT: MeritOperandType = None
    P4VA: MeritOperandType = None
    P5GT: MeritOperandType = None
    P5LT: MeritOperandType = None
    P5VA: MeritOperandType = None
    P6GT: MeritOperandType = None
    P6LT: MeritOperandType = None
    P6VA: MeritOperandType = None
    P7GT: MeritOperandType = None
    P7LT: MeritOperandType = None
    P7VA: MeritOperandType = None
    P8GT: MeritOperandType = None
    P8LT: MeritOperandType = None
    P8VA: MeritOperandType = None
    PANA: MeritOperandType = None
    PANB: MeritOperandType = None
    PANC: MeritOperandType = None
    PARA: MeritOperandType = None
    PARB: MeritOperandType = None
    PARC: MeritOperandType = None
    PARR: MeritOperandType = None
    PARX: MeritOperandType = None
    PARY: MeritOperandType = None
    PARZ: MeritOperandType = None
    PATX: MeritOperandType = None
    PATY: MeritOperandType = None
    PETC: MeritOperandType = None
    PETZ: MeritOperandType = None
    PIMH: MeritOperandType = None
    PLEN: MeritOperandType = None
    PMAG: MeritOperandType = None
    POWR: MeritOperandType = None
    PRIM: MeritOperandType = None
    PROD: MeritOperandType = None
    PMGT: MeritOperandType = None
    PMLT: MeritOperandType = None
    PMVA: MeritOperandType = None
    POPD: MeritOperandType = None
    QOAC: MeritOperandType = None
    QSUM: MeritOperandType = None
    RAGA: MeritOperandType = None
    RAGB: MeritOperandType = None
    RAGC: MeritOperandType = None
    RAGX: MeritOperandType = None
    RAGY: MeritOperandType = None
    RAGZ: MeritOperandType = None
    RAED: MeritOperandType = None
    RAEN: MeritOperandType = None
    RAID: MeritOperandType = None
    RAIN: MeritOperandType = None
    RANG: MeritOperandType = None
    REAA: MeritOperandType = None
    REAB: MeritOperandType = None
    REAC: MeritOperandType = None
    REAR: MeritOperandType = None
    REAX: MeritOperandType = None
    REAY: MeritOperandType = None
    REAZ: MeritOperandType = None
    RELI: MeritOperandType = None
    GBSR: MeritOperandType = None
    GBSS: MeritOperandType = None
    TGTH: MeritOperandType = None
    TMAS: MeritOperandType = None
    TOTR: MeritOperandType = None
    TRAC: MeritOperandType = None
    TRAD: MeritOperandType = None
    TRAE: MeritOperandType = None
    TRAI: MeritOperandType = None
    TRAN: MeritOperandType = None
    TRAR: MeritOperandType = None
    TRAX: MeritOperandType = None
    TRAY: MeritOperandType = None
    TRCX: MeritOperandType = None
    TRCY: MeritOperandType = None
    TTGT: MeritOperandType = None
    TTHI: MeritOperandType = None
    TTLT: MeritOperandType = None
    GBSW: MeritOperandType = None
    GBSP: MeritOperandType = None
    GCOS: MeritOperandType = None
    GENC: MeritOperandType = None
    GENF: MeritOperandType = None
    GLCA: MeritOperandType = None
    GLCB: MeritOperandType = None
    GLCC: MeritOperandType = None
    GLCX: MeritOperandType = None
    GLCY: MeritOperandType = None
    GLCZ: MeritOperandType = None
    GMTA: MeritOperandType = None
    GMTS: MeritOperandType = None
    GMTT: MeritOperandType = None
    GOTO: MeritOperandType = None
    GPIM: MeritOperandType = None
    GRMN: MeritOperandType = None
    GRMX: MeritOperandType = None
    GTCE: MeritOperandType = None
    HACG: MeritOperandType = None
    HHCN: MeritOperandType = None
    I1GT: MeritOperandType = None
    BIOD: MeritOperandType = None
    BIPF: MeritOperandType = None
    BLNK: MeritOperandType = None
    BSER: MeritOperandType = None
    CENX: MeritOperandType = None
    CENY: MeritOperandType = None
    CMFV: MeritOperandType = None
    CMGT: MeritOperandType = None
    CMLT: MeritOperandType = None
    CMVA: MeritOperandType = None
    CODA: MeritOperandType = None
    COGT: MeritOperandType = None
    COLT: MeritOperandType = None
    COMA: MeritOperandType = None
    CONF: MeritOperandType = None
    CONS: MeritOperandType = None
    COSI: MeritOperandType = None
    COVA: MeritOperandType = None
    CTGT: MeritOperandType = None
    CTLT: MeritOperandType = None
    I2GT: MeritOperandType = None
    I3GT: MeritOperandType = None
    I4GT: MeritOperandType = None
    I5GT: MeritOperandType = None
    I6GT: MeritOperandType = None
    I1LT: MeritOperandType = None
    I2LT: MeritOperandType = None
    I3LT: MeritOperandType = None
    I4LT: MeritOperandType = None
    I5LT: MeritOperandType = None
    I6LT: MeritOperandType = None
    I1VA: MeritOperandType = None
    I2VA: MeritOperandType = None
    I3VA: MeritOperandType = None
    I4VA: MeritOperandType = None
    I5VA: MeritOperandType = None
    I6VA: MeritOperandType = None
    IMAE: MeritOperandType = None
    IMSF: MeritOperandType = None
    INDX: MeritOperandType = None
    ISFN: MeritOperandType = None
    ISNA: MeritOperandType = None
    LACL: MeritOperandType = None
    LINV: MeritOperandType = None
    LOGE: MeritOperandType = None
    LOGT: MeritOperandType = None
    LONA: MeritOperandType = None
    LPTD: MeritOperandType = None
    MAXX: MeritOperandType = None
    MCOG: MeritOperandType = None
    MCOL: MeritOperandType = None
    MCOV: MeritOperandType = None
    MINN: MeritOperandType = None
    MNAB: MeritOperandType = None
    MNCA: MeritOperandType = None
    MNCG: MeritOperandType = None
    MNCT: MeritOperandType = None
    MNCV: MeritOperandType = None
    MNDT: MeritOperandType = None
    MNEA: MeritOperandType = None
    MNEG: MeritOperandType = None
    MNET: MeritOperandType = None
    MNIN: MeritOperandType = None
    MNPD: MeritOperandType = None
    MNSD: MeritOperandType = None
    MSWA: MeritOperandType = None
    MSWS: MeritOperandType = None
    MSWT: MeritOperandType = None
    MTFA: MeritOperandType = None
    MTFS: MeritOperandType = None
    MTFT: MeritOperandType = None
    MXAB: MeritOperandType = None
    MXCA: MeritOperandType = None
    MXCG: MeritOperandType = None
    MXCT: MeritOperandType = None
    MXCV: MeritOperandType = None
    MXDT: MeritOperandType = None
    MXEA: MeritOperandType = None
    MXEG: MeritOperandType = None
    MXET: MeritOperandType = None
    MXIN: MeritOperandType = None
    MXPD: MeritOperandType = None
    MXSD: MeritOperandType = None
    NPGT: MeritOperandType = None
    NPLT: MeritOperandType = None
    NPVA: MeritOperandType = None
    NPXG: MeritOperandType = None
    NPXL: MeritOperandType = None
    NPXV: MeritOperandType = None
    NPYG: MeritOperandType = None
    NPYL: MeritOperandType = None
    NPYV: MeritOperandType = None
    NPZG: MeritOperandType = None
    NPZL: MeritOperandType = None
    NPZV: MeritOperandType = None
    NSDC: MeritOperandType = None
    NSDD: MeritOperandType = None
    NSRA: MeritOperandType = None
    NSTR: MeritOperandType = None
    NTXG: MeritOperandType = None
    NTXL: MeritOperandType = None
    NTXV: MeritOperandType = None
    NTYG: MeritOperandType = None
    NTYL: MeritOperandType = None
    NTYV: MeritOperandType = None
    NTZG: MeritOperandType = None
    NTZL: MeritOperandType = None
    NTZV: MeritOperandType = None
    OSCD: MeritOperandType = None
    OBSN: MeritOperandType = None
    OOFF: MeritOperandType = None
    OPDC: MeritOperandType = None
    OPDM: MeritOperandType = None
    OPDX: MeritOperandType = None
    OPGT: MeritOperandType = None
    OPLT: MeritOperandType = None
    OPVA: MeritOperandType = None
    OPTH: MeritOperandType = None
    OSUM: MeritOperandType = None
    RENA: MeritOperandType = None
    RENB: MeritOperandType = None
    RENC: MeritOperandType = None
    RETX: MeritOperandType = None
    RETY: MeritOperandType = None
    RGLA: MeritOperandType = None
    RSCH: MeritOperandType = None
    RSCE: MeritOperandType = None
    RSRE: MeritOperandType = None
    RSRH: MeritOperandType = None
    RWCH: MeritOperandType = None
    RWCE: MeritOperandType = None
    RWRE: MeritOperandType = None
    RWRH: MeritOperandType = None
    SAGX: MeritOperandType = None
    SAGY: MeritOperandType = None
    SFNO: MeritOperandType = None
    SINE: MeritOperandType = None
    SKIS: MeritOperandType = None
    SKIN: MeritOperandType = None
    SPCH: MeritOperandType = None
    SPHA: MeritOperandType = None
    SQRT: MeritOperandType = None
    SSAG: MeritOperandType = None
    SUMM: MeritOperandType = None
    SVIG: MeritOperandType = None
    TANG: MeritOperandType = None
    TFNO: MeritOperandType = None
    WFNO: MeritOperandType = None
    TTVA: MeritOperandType = None
    UDOP: MeritOperandType = None
    UDOC: MeritOperandType = None
    USYM: MeritOperandType = None
    VOLU: MeritOperandType = None
    WLEN: MeritOperandType = None
    XDVA: MeritOperandType = None
    XDGT: MeritOperandType = None
    XDLT: MeritOperandType = None
    XENC: MeritOperandType = None
    XENF: MeritOperandType = None
    XNEA: MeritOperandType = None
    XNEG: MeritOperandType = None
    XNET: MeritOperandType = None
    XXEA: MeritOperandType = None
    XXEG: MeritOperandType = None
    XXET: MeritOperandType = None
    YNIP: MeritOperandType = None
    ZERN: MeritOperandType = None
    ZPLM: MeritOperandType = None
    ZTHI: MeritOperandType = None
    TOLR: MeritOperandType = None
    FTGT: MeritOperandType = None
    FTLT: MeritOperandType = None
    GLCR: MeritOperandType = None
    EFNO: MeritOperandType = None
    DIVB: MeritOperandType = None
    PROB: MeritOperandType = None
    TCGT: MeritOperandType = None
    TCLT: MeritOperandType = None
    TCVA: MeritOperandType = None
    NORX: MeritOperandType = None
    NORY: MeritOperandType = None
    NORZ: MeritOperandType = None
    NORD: MeritOperandType = None
    COSA: MeritOperandType = None
    MTHA: MeritOperandType = None
    MTHS: MeritOperandType = None
    MTHT: MeritOperandType = None
    BFSD: MeritOperandType = None
    EXPD: MeritOperandType = None
    CVIG: MeritOperandType = None
    GPSX: MeritOperandType = None
    GPSY: MeritOperandType = None
    GPRX: MeritOperandType = None
    GPRY: MeritOperandType = None
    GPRT: MeritOperandType = None
    POWF: MeritOperandType = None
    STHI: MeritOperandType = None
    CNPX: MeritOperandType = None
    CNPY: MeritOperandType = None
    CNAX: MeritOperandType = None
    CNAY: MeritOperandType = None
    STRH: MeritOperandType = None
    CIGT: MeritOperandType = None
    CILT: MeritOperandType = None
    CIVA: MeritOperandType = None
    CEGT: MeritOperandType = None
    CELT: MeritOperandType = None
    CEVA: MeritOperandType = None
    NSST: MeritOperandType = None
    ABCD: MeritOperandType = None
    DISA: MeritOperandType = None
    POWP: MeritOperandType = None
    RECI: MeritOperandType = None
    NSRM: MeritOperandType = None
    FDMO: MeritOperandType = None
    FDRE: MeritOperandType = None
    FREZ: MeritOperandType = None
    NSDE: MeritOperandType = None
    ERFP: MeritOperandType = None
    NSDP: MeritOperandType = None
    POPI: MeritOperandType = None
    ABGT: MeritOperandType = None
    ABLT: MeritOperandType = None
    SMIA: MeritOperandType = None
    SCUR: MeritOperandType = None
    SDRV: MeritOperandType = None
    REVR: MeritOperandType = None
    NSLT: MeritOperandType = None
    NSTW: MeritOperandType = None
    NSRW: MeritOperandType = None
    MNRE: MeritOperandType = None
    MXRE: MeritOperandType = None
    MNRI: MeritOperandType = None
    MXRI: MeritOperandType = None
    CEHX: MeritOperandType = None
    CEHY: MeritOperandType = None
    BLTH: MeritOperandType = None
    GBPZ: MeritOperandType = None
    NSRD: MeritOperandType = None
    NPAF: MeritOperandType = None
    MECS: MeritOperandType = None
    MECT: MeritOperandType = None
    GMTN: MeritOperandType = None
    GMTX: MeritOperandType = None
    MTFN: MeritOperandType = None
    MTFX: MeritOperandType = None
    MTHN: MeritOperandType = None
    MTHX: MeritOperandType = None
    MECA: MeritOperandType = None
    MSWN: MeritOperandType = None
    MSWX: MeritOperandType = None
    GAOI: MeritOperandType = None
    MNAI: MeritOperandType = None
    MXAI: MeritOperandType = None
    HYLD: MeritOperandType = None
    OGSS: MeritOperandType = None
    SPHD: MeritOperandType = None
    SPHS: MeritOperandType = None
    SSLP: MeritOperandType = None
    SCRV: MeritOperandType = None
    DSAG: MeritOperandType = None
    DPHS: MeritOperandType = None
    DSLP: MeritOperandType = None
    DCRV: MeritOperandType = None
    EFLA: MeritOperandType = None
    CARD: MeritOperandType = None
    TSAG: MeritOperandType = None
    PSLP: MeritOperandType = None
    QSLP: MeritOperandType = None
    RRET: MeritOperandType = None



from __future__ import annotations

from contextlib import nullcontext as does_not_raise
from datetime import datetime
from types import SimpleNamespace

import pytest

from zospy.analyses.base import AnalysisMetadata
from zospy.analyses.decorators import analysis_settings
from zospy.analyses.systemviewers import CrossSection, NSC3DLayout, NSCShadedModel, ShadedModel, Viewer3D
from zospy.analyses.systemviewers.base import SystemViewerWrapper


def assert_systemviewer_result(result, minimal_version):
    """Makes sure systemviewer results is correctly asserted for different versions of OpticStudio."""
    if minimal_version >= "24.1.0":
        assert result.data is not None
    else:  # No result.data as layout exports are not supported
        assert result.data is None


class TestBase:
    @analysis_settings
    class MockSystemViewerSettings:
        number: int = 5

    class MockSystemViewer(SystemViewerWrapper[MockSystemViewerSettings]):
        def __init__(self, *, number: int = 5, settings: TestBase.MockSystemViewerSettings | None = None):
            super().__init__(locals())

        def _create_analysis(self, *, settings_first=True):  # noqa: ARG002
            self._analysis = SimpleNamespace(
                metadata=AnalysisMetadata(DateTime=datetime.now(), LensFile="", LensTitle="", FeatureDescription=""),
                header_data=None,
                messages=[],
                Close=lambda: None,
            )

        def _do_nothing(self):
            pass

        def configure_layout_tool(self):
            pass

        def run_analysis(self) -> None:
            return None

    @pytest.mark.parametrize(
        "filename,expectation",
        [
            ("test.bmp", does_not_raise()),
            ("test.jpeg", does_not_raise()),
            ("test.png", does_not_raise()),
            ("test.jpg", pytest.raises(ValueError, match="Image file must have one of the following extensions:")),
            ("test.tiff", pytest.raises(ValueError, match="Image file must have one of the following extensions:")),
        ],
    )
    def test_validate_path(self, filename, expectation, tmp_path):
        viewer = TestBase.MockSystemViewer()

        with expectation:
            result = viewer._validate_path(tmp_path / filename)  # noqa: SLF001
            assert result == str(tmp_path / filename)

    @pytest.mark.parametrize(
        "wavelength,expected,expectation",
        [
            (1, 1, does_not_raise()),
            (4, 4, does_not_raise()),
            (-1, -1, does_not_raise()),
            ("All", -1, does_not_raise()),
            ("a", None, pytest.raises(ValueError, match="wavelength must be an integer or 'All'")),
            (0.55, None, pytest.raises(TypeError, match="wavelength must be an integer or 'All'")),
            (
                -2,
                None,
                pytest.raises(ValueError, match="wavelength must be -1 or between 1 and the number of wavelengths"),
            ),
            (
                -0,
                None,
                pytest.raises(ValueError, match="wavelength must be -1 or between 1 and the number of wavelengths"),
            ),
            (
                5,
                None,
                pytest.raises(ValueError, match="wavelength must be -1 or between 1 and the number of wavelengths"),
            ),
        ],
    )
    def test_validate_wavelength(self, wavelength, expected, expectation, simple_system):
        while simple_system.SystemData.Wavelengths.NumberOfWavelengths < 4:
            simple_system.SystemData.Wavelengths.AddWavelength(0.555, 1)

        viewer = TestBase.MockSystemViewer()
        viewer.run(simple_system)

        with expectation:
            result = viewer._validate_wavelength(wavelength)  # noqa: SLF001
            assert result == expected

    @pytest.mark.parametrize(
        "field,expected,expectation",
        [
            (1, 1, does_not_raise()),
            (4, 4, does_not_raise()),
            (-1, -1, does_not_raise()),
            ("All", -1, does_not_raise()),
            ("a", None, pytest.raises(ValueError, match="field must be an integer or 'All'")),
            (0.55, None, pytest.raises(TypeError, match="field must be an integer or 'All'")),
            (
                -2,
                None,
                pytest.raises(ValueError, match="field must be -1 or between 1 and the number of fields"),
            ),
            (0, None, pytest.raises(ValueError, match="field must be -1 or between 1 and the number of fields")),
            (
                5,
                None,
                pytest.raises(ValueError, match="field must be -1 or between 1 and the number of fields"),
            ),
        ],
    )
    def test_validate_field(self, field, expected, expectation, simple_system):
        while simple_system.SystemData.Fields.NumberOfFields < 4:
            simple_system.SystemData.Fields.AddField(1, 1, 1)

        viewer = TestBase.MockSystemViewer()
        viewer.run(simple_system)

        with expectation:
            result = viewer._validate_field(field)  # noqa: SLF001
            assert result == expected

    @pytest.mark.parametrize(
        "start_surface,end_surface,expected,expectation",
        [
            (1, 2, 2, does_not_raise()),
            (1, -1, 4, does_not_raise()),
            (
                1,
                5,
                None,
                pytest.raises(
                    ValueError,
                    match="end_surface must be -1 or greater than start_surface and less than the number of surfaces",
                ),
            ),
            (
                3,
                1,
                None,
                pytest.raises(
                    ValueError,
                    match="end_surface must be -1 or greater than start_surface and less than the number of surfaces",
                ),
            ),
        ],
    )
    def test_validate_end_surface(self, start_surface, end_surface, expected, expectation, simple_system):
        viewer = TestBase.MockSystemViewer()
        viewer.run(simple_system)

        with expectation:
            result = viewer._validate_end_surface(start_surface, end_surface)  # noqa: SLF001
            assert result == expected

    @pytest.mark.skip_for_opticstudio_versions(">=24.1.0", "Settings are supported from OpticStudio 24R1")
    def test_warn_ignored_settings(self, simple_system):
        viewer = TestBase.MockSystemViewer(number=6)

        with pytest.warns(UserWarning, match="Some parameters were specified but ignored"):
            viewer.run(simple_system)


class TestCrossSection:
    def test_can_run(self, simple_system, optic_studio_version):
        result = CrossSection().run(simple_system)
        assert_systemviewer_result(result, optic_studio_version)

    def test_to_json(self, simple_system):
        result = CrossSection().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()


class TestViewer3D:
    def test_can_run(self, simple_system, optic_studio_version):
        result = Viewer3D().run(simple_system)
        assert_systemviewer_result(result, optic_studio_version)

    def test_to_json(self, simple_system):
        result = Viewer3D().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()


class TestShadedModel:
    def test_can_run(self, simple_system, optic_studio_version):
        result = ShadedModel().run(simple_system)
        assert_systemviewer_result(result, optic_studio_version)

    def test_to_json(self, simple_system):
        result = ShadedModel().run(simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()


class TestNSC3DLayout:
    def test_can_run(self, nsc_simple_system, optic_studio_version):
        result = NSC3DLayout().run(nsc_simple_system)
        assert_systemviewer_result(result, optic_studio_version)

    def test_to_json(self, nsc_simple_system):
        result = NSC3DLayout().run(nsc_simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()


class TestNSCShadedModel:
    def test_can_run(self, nsc_simple_system, optic_studio_version):
        result = NSCShadedModel().run(nsc_simple_system)
        assert_systemviewer_result(result, optic_studio_version)

    def test_to_json(self, nsc_simple_system):
        result = NSCShadedModel().run(nsc_simple_system)
        assert result.from_json(result.to_json()).to_json() == result.to_json()



"""Base classes and functions for OpticStudio analysis output parsers."""

from __future__ import annotations

from functools import lru_cache
from typing import Any

from lark import Lark, Transformer
from lark.visitors import merge_transformers

from zospy.analyses.parsers.transformers import ZospyTransformer


@lru_cache
def load_grammar(name: str) -> Lark:
    """Load a grammar file and return a Lark parser.

    Grammars are loaded from `zospy.analyses.new.parsers.grammars` package.
    Grammar file names must end with `.lark`.

    Parameters
    ----------
    name : str
        The name of the grammar file to load, without the `.lark` extension.

    Returns
    -------
    Lark
        A Lark parser object.

    Raises
    ------
    FileNotFoundError
        If the grammar file is not found.
    """
    try:
        return Lark.open_from_package("zospy.analyses.parsers.grammars", f"{name}.lark", parser="earley", start="start")
    except (FileNotFoundError, OSError) as e:
        raise FileNotFoundError(f"Grammar file {name}.lark not found") from e


def parse(text: str, parser: Lark, transformer: type[Transformer]) -> dict[str, dict[list[int | float], Any]]:
    """Parse text using a Lark parser and a transformer.

    Parameters
    ----------
    text : str
        The text to parse.
    parser : Lark
        The Lark parser object. Use `load_grammar` to load a grammar file.
    transformer : type[Transformer]
        The transformer class to use.

    Returns
    -------
    dict[str, dict[list[int | float], Any]]
        The text output parsed into a dictionary.
    """
    merged_transformer = merge_transformers(transformer(), zospy=ZospyTransformer())

    return merged_transformer.transform(parser.parse(text))



import pytest
from pandas.testing import assert_frame_equal

from zospy.analyses.old.surface import curvature

pytestmark = pytest.mark.old_analyses


class TestCurvature:
    SKIP_SAGITTAL_CURVATURE = "SagitalCurvature is renamed to SagittalCurvature in OpticStudio 24.1.2 and higher"

    def test_can_run_curvature(self, simple_system):
        result = curvature(simple_system, surface=2)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = curvature(simple_system, surface=2)

        assert result.from_json(result.to_json())

    @pytest.mark.parametrize(
        "sampling,data,remove,surface,showas,contourformat,bfs_criterion,bfs_reversedirection",
        [
            ("65x65", "TangentialCurvature", None, 2, "Surface", "", "MinimumVolume", False),
            pytest.param(
                "129x129",
                "SagitalCurvature",
                None,
                2,
                "Contour",
                "",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions(">=24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagitalCurvature",
                "BaseROC",
                2,
                "Contour",
                "0.1",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions(">=24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagittalCurvature",
                None,
                2,
                "Contour",
                "",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions("<24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagittalCurvature",
                "BaseROC",
                2,
                "Contour",
                "0.1",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions("<24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "", "MinimumRMS", False),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "", "MinimumVolume", False),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "0.2", "MinimumVolume", True),
        ],
    )
    def test_curvature_returns_correct_result(
        self,
        simple_system,
        sampling,
        data,
        remove,
        surface,
        showas,
        contourformat,
        bfs_criterion,
        bfs_reversedirection,
        expected_data,
    ):
        result = curvature(
            simple_system,
            sampling=sampling,
            data=data,
            remove=remove,
            surface=surface,
            showas=showas,
            contourformat=contourformat,
            bfs_criterion=bfs_criterion,
            bfs_reversedirection=bfs_reversedirection,
        )

        assert_frame_equal(result.Data, expected_data.Data)

    @pytest.mark.parametrize(
        "sampling,data,remove,surface,showas,contourformat,bfs_criterion,bfs_reversedirection",
        [
            ("65x65", "TangentialCurvature", None, 2, "Surface", "", "MinimumVolume", False),
            pytest.param(
                "129x129",
                "SagitalCurvature",
                None,
                2,
                "Contour",
                "",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions(">=24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagitalCurvature",
                "BaseROC",
                2,
                "Contour",
                "0.1",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions(">=24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagittalCurvature",
                None,
                2,
                "Contour",
                "",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions("<24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            pytest.param(
                "129x129",
                "SagittalCurvature",
                "BaseROC",
                2,
                "Contour",
                "0.1",
                "MinimumVolume",
                False,
                marks=pytest.mark.skip_for_opticstudio_versions("<24.1.2", SKIP_SAGITTAL_CURVATURE),
            ),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "", "MinimumRMS", False),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "", "MinimumVolume", False),
            ("33x33", "X_Curvature", "BestFitSphere", 3, "Contour", "0.2", "MinimumVolume", True),
        ],
    )
    def test_curvature_matches_reference_data(
        self,
        simple_system,
        sampling,
        data,
        remove,
        surface,
        showas,
        contourformat,
        bfs_criterion,
        bfs_reversedirection,
        reference_data,
    ):
        result = curvature(
            simple_system,
            sampling=sampling,
            data=data,
            remove=remove,
            surface=surface,
            showas=showas,
            contourformat=contourformat,
            bfs_criterion=bfs_criterion,
            bfs_reversedirection=bfs_reversedirection,
        )

        assert_frame_equal(result.Data, reference_data.Data)



import pytest

from tests.config import REFERENCE_DATA_FOLDER, REFERENCE_VERSION
from zospy.analyses.old.base import AnalysisResult


@pytest.fixture
def expected_data(request, optic_studio_version) -> AnalysisResult:
    filename = f"{optic_studio_version}-{request.fspath.basename}-{request.node.name}"

    data_file = request.config.rootpath / REFERENCE_DATA_FOLDER / "old" / f"{filename}.json"

    if not data_file.exists():
        pytest.skip(f"Data file {data_file} does not exist")

    with open(data_file) as f:
        return AnalysisResult.from_json(f.read())


@pytest.fixture
def reference_data(request) -> AnalysisResult:
    filename = f"{REFERENCE_VERSION}-{request.fspath.basename}-{request.node.name}"
    filename = filename.replace("matches_reference_data", "returns_correct_result")

    data_file = request.config.rootpath / REFERENCE_DATA_FOLDER / "old" / f"{filename}.json"

    if not data_file.exists():
        pytest.skip(f"Data file {data_file} does not exist")

    with open(data_file) as f:
        return AnalysisResult.from_json(f.read())



"""OpticStudio analyses from the Modulation Transfer Function (MTF) category."""

from zospy.analyses.mtf.fft_through_focus_mtf import FFTThroughFocusMTF, FFTThroughFocusMTFSettings
from zospy.analyses.mtf.huygens_mtf import HuygensMTF, HuygensMtfSettings

__all__ = ("FFTThroughFocusMTF", "FFTThroughFocusMTFSettings", "HuygensMTF", "HuygensMtfSettings")
